# -*- coding: utf-8 -*-
'''
Tests for the Openstack Cloud Provider
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import logging
import os

# Import Salt Testing libs
from tests.support.case import ModuleCase, ShellCase
from tests.support.paths import FILES
from tests.support.unit import skipIf
from tests.support.helpers import destructiveTest, expensiveTest, generate_random_name
from tests.support.mixins import SaltReturnAssertsMixin

# Import Salt Libs
from salt.config import cloud_providers_config

log = logging.getLogger(__name__)

try:
    import keystoneclient  # pylint: disable=import-error,unused-import
    from libcloud.common.openstack_identity import OpenStackIdentity_3_0_Connection
    from libcloud.common.openstack_identity import OpenStackIdentityTokenScope
    HAS_KEYSTONE = True
except ImportError:
    HAS_KEYSTONE = False

# Import Third-Party Libs
try:
    import shade  # pylint: disable=unused-import
    HAS_SHADE = True
except ImportError:
    HAS_SHADE = False

# Create the cloud instance name to be used throughout the tests
INSTANCE_NAME = generate_random_name('CLOUD-TEST-')
PROVIDER_NAME = 'openstack'
DRIVER_NAME = 'openstack'


@skipIf(
    not HAS_KEYSTONE,
    'Please install keystoneclient and a keystone server before running'
    'openstack integration tests.'
)
class OpenstackTest(ModuleCase, SaltReturnAssertsMixin):
    '''
    Validate the keystone state
    '''
    endpoint = 'http://localhost:35357/v2.0'
    token = 'administrator'

    @destructiveTest
    def test_aaa_setup_keystone_endpoint(self):
        ret = self.run_state('keystone.service_present',
                             name='keystone',
                             description='OpenStack Identity',
                             service_type='identity',
                             connection_endpoint=self.endpoint,
                             connection_token=self.token)
        self.assertTrue(ret['keystone_|-keystone_|-keystone_|-service_present']['result'])

        ret = self.run_state('keystone.endpoint_present',
                             name='keystone',
                             region='RegionOne',
                             publicurl='http://localhost:5000/v2.0',
                             internalurl='http://localhost:5000/v2.0',
                             adminurl='http://localhost:35357/v2.0',
                             connection_endpoint=self.endpoint,
                             connection_token=self.token)
        self.assertTrue(ret['keystone_|-keystone_|-keystone_|-endpoint_present']['result'])

        ret = self.run_state('keystone.tenant_present',
                             name='admin',
                             description='Admin Project',
                             connection_endpoint=self.endpoint,
                             connection_token=self.token)
        self.assertTrue(ret['keystone_|-admin_|-admin_|-tenant_present']['result'])

        ret = self.run_state('keystone.tenant_present',
                             name='demo',
                             description='Demo Project',
                             connection_endpoint=self.endpoint,
                             connection_token=self.token)
        self.assertTrue(ret['keystone_|-demo_|-demo_|-tenant_present']['result'])

        ret = self.run_state('keystone.role_present',
                             name='admin',
                             connection_endpoint=self.endpoint,
                             connection_token=self.token)
        self.assertTrue(ret['keystone_|-admin_|-admin_|-role_present']['result'])

        ret = self.run_state('keystone.role_present',
                             name='user',
                             connection_endpoint=self.endpoint,
                             connection_token=self.token)
        self.assertTrue(ret['keystone_|-user_|-user_|-role_present']['result'])

        ret = self.run_state('keystone.user_present',
                             name='admin',
                             email='admin@example.com',
                             password='adminpass',
                             tenant='admin',
                             roles={'admin': ['admin']},
                             connection_endpoint=self.endpoint,
                             connection_token=self.token)
        self.assertTrue(ret['keystone_|-admin_|-admin_|-user_present']['result'])

        ret = self.run_state('keystone.user_present',
                             name='demo',
                             email='demo@example.com',
                             password='demopass',
                             tenant='demo',
                             roles={'demo': ['user']},
                             connection_endpoint=self.endpoint,
                             connection_token=self.token)
        self.assertTrue(ret['keystone_|-demo_|-demo_|-user_present']['result'])

    @destructiveTest
    def test_zzz_teardown_keystone_endpoint(self):
        ret = self.run_state('keystone.user_absent',
                             name='admin',
                             connection_endpoint=self.endpoint,
                             connection_token=self.token)
        self.assertTrue(ret['keystone_|-admin_|-admin_|-user_absent']['result'])

        ret = self.run_state('keystone.user_absent',
                             name='demo',
                             connection_endpoint=self.endpoint,
                             connection_token=self.token)
        self.assertTrue(ret['keystone_|-demo_|-demo_|-user_absent']['result'])

        ret = self.run_state('keystone.role_absent',
                             name='admin',
                             connection_endpoint=self.endpoint,
                             connection_token=self.token)
        self.assertTrue(ret['keystone_|-admin_|-admin_|-role_absent']['result'])

        ret = self.run_state('keystone.role_absent',
                             name='user',
                             connection_endpoint=self.endpoint,
                             connection_token=self.token)
        self.assertTrue(ret['keystone_|-user_|-user_|-role_absent']['result'])

        ret = self.run_state('keystone.tenant_absent',
                             name='admin',
                             connection_endpoint=self.endpoint,
                             connection_token=self.token)
        self.assertTrue(ret['keystone_|-admin_|-admin_|-tenant_absent']['result'])

        ret = self.run_state('keystone.tenant_absent',
                             name='demo',
                             connection_endpoint=self.endpoint,
                             connection_token=self.token)
        self.assertTrue(ret['keystone_|-demo_|-demo_|-tenant_absent']['result'])

        ret = self.run_state('keystone.service_absent',
                             name='keystone',
                             connection_endpoint=self.endpoint,
                             connection_token=self.token)
        self.assertTrue(ret['keystone_|-keystone_|-keystone_|-service_absent']['result'])

    @destructiveTest
    def test_libcloud_auth_v3(self):
        driver = OpenStackIdentity_3_0_Connection(auth_url='http://localhost:5000',
                                                  user_id='admin',
                                                  key='adminpass',
                                                  token_scope=OpenStackIdentityTokenScope.PROJECT,
                                                  domain_name='Default',
                                                  tenant_name='admin')
        driver.authenticate()
        self.assertTrue(driver.auth_token)


@skipIf(not HAS_SHADE, 'openstack driver requires `shade`')
class RackspaceTest(ShellCase):
    '''
    Integration tests for the Rackspace cloud provider using the Openstack driver
    '''

    @expensiveTest
    def setUp(self):
        '''
        Sets up the test requirements
        '''
        super(RackspaceTest, self).setUp()

        # check if appropriate cloud provider and profile files are present
        profile_str = 'openstack-config'
        providers = self.run_cloud('--list-providers')
        if profile_str + ':' not in providers:
            self.skipTest(
                'Configuration file for {0} was not found. Check {0}.conf files '
                'in tests/integration/files/conf/cloud.*.d/ to run these tests.'
                .format(PROVIDER_NAME)
            )

        # check if personal access token, ssh_key_file, and ssh_key_names are present
        config = cloud_providers_config(
            os.path.join(
                FILES,
                'conf',
                'cloud.providers.d',
                PROVIDER_NAME + '.conf'
            )
        )

        region_name = config[profile_str][DRIVER_NAME].get('region_name')
        auth = config[profile_str][DRIVER_NAME].get('auth')
        cloud = config[profile_str][DRIVER_NAME].get('cloud')
        if not region_name or not (auth or cloud):
            self.skipTest(
                'A region_name and (auth or cloud) must be provided to run these '
                'tests. Check tests/integration/files/conf/cloud.providers.d/{0}.conf'
                .format(PROVIDER_NAME)
            )

    def test_instance(self):
        '''
        Test creating an instance on rackspace with the openstack driver
        '''
        # check if instance with salt installed returned
        try:
            self.assertIn(
                INSTANCE_NAME,
                [i.strip() for i in self.run_cloud('-p rackspace-test {0}'.format(INSTANCE_NAME), timeout=500)]
            )
        except AssertionError:
            self.run_cloud('-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=500)
            raise

        # delete the instance
        try:
            self.assertIn(
                INSTANCE_NAME + ':',
                [i.strip() for i in self.run_cloud('-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=500)]
            )
        except AssertionError:
            raise

    def tearDown(self):
        '''
        Clean up after tests
        '''
        query = self.run_cloud('--query')
        ret = '        {0}:'.format(INSTANCE_NAME)

        # if test instance is still present, delete it
        if ret in query:
            self.run_cloud('-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=500)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Anthony Shaw <anthonyshaw@apache.org>
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON
)

import tests.support.napalm as napalm_test_support
import salt.modules.napalm_network as napalm_network  # NOQA


@skipIf(NO_MOCK, NO_MOCK_REASON)
class NapalmNetworkModuleTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        module_globals = {
            '__salt__': {
                'config.option': MagicMock(return_value={
                    'test': {
                        'driver': 'test',
                        'key': '2orgk34kgk34g'
                    }
                }),
                'file.file_exists': napalm_test_support.true,
                'file.join': napalm_test_support.join,
                'file.get_managed': napalm_test_support.get_managed_file,
                'random.hash': napalm_test_support.random_hash
            }
        }

        return {napalm_network: module_globals}

    def test_connected_pass(self):
        ret = napalm_network.connected()
        assert ret['out'] is True

    def test_facts(self):
        ret = napalm_network.facts()
        assert ret['out'] == napalm_test_support.TEST_FACTS

    def test_environment(self):
        ret = napalm_network.environment()
        assert ret['out'] == napalm_test_support.TEST_ENVIRONMENT

    def test_cli_single_command(self):
        '''
        Test that CLI works with 1 arg
        '''
        ret = napalm_network.cli("show run")
        assert ret['out'] == napalm_test_support.TEST_COMMAND_RESPONSE

    def test_cli_multi_command(self):
        '''
        Test that CLI works with 2 arg
        '''
        ret = napalm_network.cli("show run", "show run")
        assert ret['out'] == napalm_test_support.TEST_COMMAND_RESPONSE

    def test_traceroute(self):
        ret = napalm_network.traceroute('destination.com')
        assert list(ret['out'].keys())[0] == 'success'

    def test_ping(self):
        ret = napalm_network.ping('destination.com')
        assert list(ret['out'].keys())[0] == 'success'

    def test_arp(self):
        ret = napalm_network.arp()
        assert ret['out'] == napalm_test_support.TEST_ARP_TABLE

    def test_ipaddrs(self):
        ret = napalm_network.ipaddrs()
        assert ret['out'] == napalm_test_support.TEST_IPADDRS

    def test_interfaces(self):
        ret = napalm_network.interfaces()
        assert ret['out'] == napalm_test_support.TEST_INTERFACES

    def test_lldp(self):
        ret = napalm_network.lldp()
        assert ret['out'] == napalm_test_support.TEST_LLDP_NEIGHBORS

    def test_mac(self):
        ret = napalm_network.mac()
        assert ret['out'] == napalm_test_support.TEST_MAC_TABLE

    def test_config(self):
        ret = napalm_network.config('running')
        assert ret['out'] == napalm_test_support.TEST_RUNNING_CONFIG

    def test_optics(self):
        ret = napalm_network.optics()
        assert ret['out'] == napalm_test_support.TEST_OPTICS

    def test_load_config(self):
        ret = napalm_network.load_config(text='new config')
        assert ret['result']

    def test_load_config_replace(self):
        ret = napalm_network.load_config(text='new config', replace=True)
        assert ret['result']

    def test_load_template(self):
        ret = napalm_network.load_template('set_ntp_peers',
                                           peers=['192.168.0.1'])
        assert ret['out'] is None

    def test_commit(self):
        ret = napalm_network.commit()
        assert ret['out'] == napalm_test_support.TEST_RUNNING_CONFIG

    def test_discard_config(self):
        ret = napalm_network.discard_config()
        assert ret['out'] == napalm_test_support.TEST_RUNNING_CONFIG

    def test_compare_config(self):
        ret = napalm_network.compare_config()
        assert ret['out'] == napalm_test_support.TEST_RUNNING_CONFIG

    def test_rollback(self):
        ret = napalm_network.rollback()
        assert ret['out'] == napalm_test_support.TEST_RUNNING_CONFIG

    def test_config_changed(self):
        ret = napalm_network.config_changed()
        assert ret == (True, '')

    def test_config_control(self):
        ret = napalm_network.config_control()
        assert ret == (True, '')



# -*- coding: utf-8 -*-
'''
Tests for the spm build utility
'''
# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import shutil

# Import Salt libs
import salt.utils.files
import salt.utils.path

# Import Salt Testing libs
from tests.support.case import SPMCase, ModuleCase
from tests.support.helpers import destructiveTest
from tests.support.unit import skipIf


@destructiveTest
class SPMBuildTest(SPMCase, ModuleCase):
    '''
    Validate the spm build command
    '''
    def setUp(self):
        self.config = self._spm_config()
        self._spm_build_files(self.config)

    def test_spm_build(self):
        '''
        test spm build
        '''
        self.run_spm('build', self.config, self.formula_dir)
        spm_file = os.path.join(self.config['spm_build_dir'], 'apache-201506-2.spm')
        # Make sure .spm file gets created
        self.assertTrue(os.path.exists(spm_file))
        # Make sure formula path dir is created
        self.assertTrue(os.path.isdir(self.config['formula_path']))

    @skipIf(salt.utils.path.which('fallocate') is None, 'fallocate not installed')
    def test_spm_build_big_file(self):
        '''
        test spm build with a big file
        '''
        # check to make sure there is enough space to run this test
        check_space = self.run_function('status.diskusage', ['/tmp'])
        space = check_space['/tmp']['available']
        if space < 3000000000:
            self.skipTest('Not enough space on host to run this test')

        self.run_function('cmd.run',
                          ['fallocate -l 1G {0}'.format(os.path.join(self.formula_sls_dir,
                                                                     'bigfile.txt'))])
        self.run_spm('build', self.config, self.formula_dir)
        spm_file = os.path.join(self.config['spm_build_dir'], 'apache-201506-2.spm')
        self.run_spm('install', self.config, spm_file)

        get_files = self.run_spm('files', self.config, 'apache')

        files = ['apache.sls', 'bigfile.txt']
        for sls in files:
            self.assertIn(sls, ' '.join(get_files))

    def test_spm_build_exclude(self):
        '''
        test spm build while excluding directory
        '''
        git_dir = os.path.join(self.formula_sls_dir, '.git')
        os.makedirs(git_dir)
        files = ['donotbuild1', 'donotbuild2', 'donotbuild3']

        for git_file in files:
            with salt.utils.files.fopen(os.path.join(git_dir, git_file), 'w') as fp:
                fp.write('Please do not include me in build')

        self.run_spm('build', self.config, self.formula_dir)
        spm_file = os.path.join(self.config['spm_build_dir'], 'apache-201506-2.spm')
        self.run_spm('install', self.config, spm_file)

        get_files = self.run_spm('files', self.config, 'apache')

        for git_file in files:
            self.assertNotIn(git_file, ' '.join(get_files))

    def tearDown(self):
        shutil.rmtree(self._tmp_spm)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.states.win_servermanager as win_servermanager


@skipIf(NO_MOCK, NO_MOCK_REASON)
class WinServermanagerTestCase(TestCase, LoaderModuleMockMixin):
    '''
        Validate the win_servermanager state
    '''
    def setup_loader_modules(self):
        return {win_servermanager: {}}

    def test_installed(self):
        '''
            Test to install the windows feature
        '''
        mock_list = MagicMock(
            side_effect=[{'spongebob': 'squarepants'},
                         {'squidward': 'patrick'},
                         {'spongebob': 'squarepants'},
                         {'spongebob': 'squarepants',
                          'squidward': 'patrick'}])
        mock_install = MagicMock(
            return_value={'Success': True,
                          'Restarted': False,
                          'RestartNeeded': False,
                          'ExitCode': 1234,
                          'Features': {
                              'squidward': {
                                  'DisplayName': 'Squidward',
                                  'Message': '',
                                  'RestartNeeded': True,
                                  'SkipReason': 0,
                                  'Success': True
                              }
                          }})
        with patch.dict(win_servermanager.__salt__,
                        {"win_servermanager.list_installed": mock_list,
                         "win_servermanager.install": mock_install}):
            ret = {'name': 'spongebob',
                   'changes': {},
                   'result': True,
                   'comment': 'The following features are already installed:\n'
                              '- spongebob'}
            self.assertDictEqual(win_servermanager.installed('spongebob'), ret)

            with patch.dict(win_servermanager.__opts__, {"test": True}):
                ret = {'name': 'spongebob',
                       'result': None,
                       'comment': '',
                       'changes': {
                           'spongebob': 'Will be installed recurse=False'}}
                self.assertDictEqual(
                    win_servermanager.installed('spongebob'), ret)

            with patch.dict(win_servermanager.__opts__, {"test": False}):
                ret = {'name': 'squidward',
                       'result': True,
                       'comment': 'Installed the following:\n- squidward',
                       'changes': {
                           'squidward': {'new': 'patrick', 'old': ''}}}
                self.assertDictEqual(
                    win_servermanager.installed('squidward'), ret)

    def test_removed(self):
        '''
            Test to remove the windows feature
        '''
        mock_list = MagicMock(
            side_effect=[{'spongebob': 'squarepants'},
                         {'squidward': 'patrick'},
                         {'spongebob': 'squarepants',
                          'squidward': 'patrick'},
                         {'spongebob': 'squarepants'}])
        mock_remove = MagicMock(
            return_value={'Success': True,
                          'RestartNeeded': False,
                          'Restarted': False,
                          'ExitCode': 1234,
                          'Features': {
                              'squidward': {
                                  'DisplayName': 'Squidward',
                                  'Message': '',
                                  'RestartNeeded': True,
                                  'SkipReason': 0,
                                  'Success': True
                              }
                          }})
        with patch.dict(win_servermanager.__salt__,
                        {"win_servermanager.list_installed": mock_list,
                         "win_servermanager.remove": mock_remove}):
            ret = {'name': 'squidward',
                   'changes': {},
                   'result': True,
                   'comment': 'The following features are not installed:\n'
                              '- squidward'}
            self.assertDictEqual(
                win_servermanager.removed('squidward'), ret)

            with patch.dict(win_servermanager.__opts__, {"test": True}):
                ret = {'name': 'squidward',
                       'result': None,
                       'comment': '',
                       'changes': {'squidward': 'Will be removed'}}
                self.assertDictEqual(
                    win_servermanager.removed('squidward'), ret)

            with patch.dict(win_servermanager.__opts__, {"test": False}):
                ret = {'name': 'squidward',
                       'result': True,
                       'comment': 'Removed the following:\n- squidward',
                       'changes': {
                           'squidward': {'new': '', 'old': 'patrick'}}}
                self.assertDictEqual(
                    win_servermanager.removed('squidward'), ret)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Alexandru Bleotu <alexandru.bleotu@morganstanley.com>`

    Tests for functions in salt.states.esxdatacenter
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.esxdatacenter as esxdatacenter
from salt.exceptions import CommandExecutionError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class DatacenterConfiguredTestCase(TestCase, LoaderModuleMockMixin):
    '''Tests for salt.modules.esxdatacenter.datacenter_configured'''

    def setup_loader_modules(self):
        return {
            esxdatacenter: {
                '__virtual__': MagicMock(return_value='esxdatacenter')}}

    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_dc = MagicMock()

        patcher = patch.dict(
            esxdatacenter.__salt__,
            {'vsphere.get_proxy_type': MagicMock(),
             'vsphere.get_service_instance_via_proxy':
             MagicMock(return_value=self.mock_si),
             'vsphere.list_datacenters_via_proxy':
             MagicMock(return_value=[self.mock_dc]),
             'vsphere.disconnect': MagicMock()})
        patcher.start()
        self.addCleanup(patcher.stop)
        patcher = patch.dict(esxdatacenter.__opts__, {'test': False})
        patcher.start()
        self.addCleanup(patcher.stop)

    def tearDown(self):
        for attrname in ('mock_si',):
            try:
                delattr(self, attrname)
            except AttributeError:
                continue

    def test_dc_name_different_proxy(self):
        with patch.dict(esxdatacenter.__salt__,
                        {'vsphere.get_proxy_type':
                         MagicMock(return_value='different_proxy')}):
            res = esxdatacenter.datacenter_configured('fake_dc')
        self.assertDictEqual(res, {'name': 'fake_dc',
                                   'changes': {},
                                   'pchanges': {},
                                   'result': True,
                                   'comment': 'Datacenter \'fake_dc\' already '
                                   'exists. Nothing to be done.'})

    def test_dc_name_esxdatacenter_proxy(self):
        with patch.dict(esxdatacenter.__salt__,
                        {'vsphere.get_proxy_type':
                         MagicMock(return_value='esxdatacenter'),
                         'esxdatacenter.get_details':
                         MagicMock(return_value={'datacenter': 'proxy_dc'})}):
            res = esxdatacenter.datacenter_configured('fake_dc')
        self.assertDictEqual(res, {'name': 'fake_dc',
                                   'changes': {},
                                   'pchanges': {},
                                   'result': True,
                                   'comment': 'Datacenter \'proxy_dc\' '
                                   'already exists. Nothing to be done.'})

    def test_get_service_instance(self):
        mock_get_service_instance = MagicMock()
        with patch.dict(esxdatacenter.__salt__,
                        {'vsphere.get_service_instance_via_proxy':
                         mock_get_service_instance}):
            esxdatacenter.datacenter_configured('fake_dc')
        mock_get_service_instance.assert_called_once_with()

    def test_list_datacenters(self):
        mock_list_datacenters = MagicMock()
        with patch.dict(esxdatacenter.__salt__,
                        {'vsphere.list_datacenters_via_proxy':
                         mock_list_datacenters}):
            esxdatacenter.datacenter_configured('fake_dc')
        mock_list_datacenters.assert_called_once_with(
            datacenter_names=['fake_dc'], service_instance=self.mock_si)

    def test_create_datacenter(self):
        mock_create_datacenter = MagicMock()
        with patch.dict(esxdatacenter.__salt__,
                        {'vsphere.list_datacenters_via_proxy':
                         MagicMock(return_value=[]),
                         'vsphere.create_datacenter':
                         mock_create_datacenter}):
            res = esxdatacenter.datacenter_configured('fake_dc')
        mock_create_datacenter.assert_called_once_with('fake_dc', self.mock_si)
        self.assertDictEqual(res,
                             {'name': 'fake_dc',
                              'changes': {'new': {'name': 'fake_dc'}},
                              'pchanges': {},
                              'result': True,
                              'comment': 'Created datacenter \'fake_dc\'.'})

    def test_create_datacenter_test_mode(self):
        with patch.dict(esxdatacenter.__opts__, {'test': True}):
            with patch.dict(esxdatacenter.__salt__,
                            {'vsphere.list_datacenters_via_proxy':
                             MagicMock(return_value=[])}):
                res = esxdatacenter.datacenter_configured('fake_dc')
        self.assertDictEqual(res,
                             {'name': 'fake_dc',
                              'changes': {},
                              'pchanges': {'new': {'name': 'fake_dc'}},
                              'result': None,
                              'comment': 'State will create '
                              'datacenter \'fake_dc\'.'})

    def test_nothing_to_be_done_test_mode(self):
        with patch.dict(esxdatacenter.__opts__, {'test': True}):
            with patch.dict(esxdatacenter.__salt__,
                            {'vsphere.get_proxy_type':
                             MagicMock(return_value='different_proxy')}):
                res = esxdatacenter.datacenter_configured('fake_dc')
        self.assertDictEqual(res, {'name': 'fake_dc',
                                   'changes': {},
                                   'pchanges': {},
                                   'result': True,
                                   'comment': 'Datacenter \'fake_dc\' already '
                                   'exists. Nothing to be done.'})

    def test_state_get_service_instance_raise_command_execution_error(self):
        mock_disconnect = MagicMock()
        with patch.dict(esxdatacenter.__salt__,
                        {'vsphere.disconnect': mock_disconnect,
                         'vsphere.get_service_instance_via_proxy':
                         MagicMock(
                             side_effect=CommandExecutionError('Error'))}):
            res = esxdatacenter.datacenter_configured('fake_dc')
        self.assertEqual(mock_disconnect.call_count, 0)
        self.assertDictEqual(res, {'name': 'fake_dc',
                                   'changes': {},
                                   'pchanges': {},
                                   'result': False,
                                   'comment': 'Error'})

    def test_state_raise_command_execution_error_after_si(self):
        mock_disconnect = MagicMock()
        with patch.dict(esxdatacenter.__salt__,
                        {'vsphere.disconnect': mock_disconnect,
                         'vsphere.list_datacenters_via_proxy':
                         MagicMock(
                             side_effect=CommandExecutionError('Error'))}):
            res = esxdatacenter.datacenter_configured('fake_dc')
        mock_disconnect.assert_called_once_with(self.mock_si)
        self.assertDictEqual(res, {'name': 'fake_dc',
                                   'changes': {},
                                   'pchanges': {},
                                   'result': False,
                                   'comment': 'Error'})

    def test_state_raise_command_execution_error_test_mode(self):
        with patch.dict(esxdatacenter.__opts__, {'test': True}):
            with patch.dict(esxdatacenter.__salt__,
                            {'vsphere.list_datacenters_via_proxy':
                             MagicMock(
                                 side_effect=CommandExecutionError('Error'))}):
                res = esxdatacenter.datacenter_configured('fake_dc')
        self.assertDictEqual(res, {'name': 'fake_dc',
                                   'changes': {},
                                   'pchanges': {},
                                   'result': None,
                                   'comment': 'Error'})



# -*- coding: utf-8 -*-
'''
Tests for salt.utils.jid
'''

# Import Python libs
from __future__ import absolute_import, unicode_literals
import datetime
import os

# Import Salt libs
import salt.utils.jid
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)


class JidTestCase(TestCase):
    def test_jid_to_time(self):
        test_jid = 20131219110700123489
        expected_jid = '2013, Dec 19 11:07:00.123489'
        self.assertEqual(salt.utils.jid.jid_to_time(test_jid), expected_jid)

        # Test incorrect lengths
        incorrect_jid_length = 2012
        self.assertEqual(salt.utils.jid.jid_to_time(incorrect_jid_length), '')

    def test_is_jid(self):
        self.assertTrue(salt.utils.jid.is_jid('20131219110700123489'))  # Valid JID
        self.assertFalse(salt.utils.jid.is_jid(20131219110700123489))  # int
        self.assertFalse(salt.utils.jid.is_jid('2013121911070012348911111'))  # Wrong length

    @skipIf(NO_MOCK, NO_MOCK_REASON)
    def test_gen_jid(self):
        now = datetime.datetime(2002, 12, 25, 12, 00, 00, 00)
        with patch('datetime.datetime'):
            datetime.datetime.now.return_value = now
            ret = salt.utils.jid.gen_jid({})
            self.assertEqual(ret, '20021225120000000000')
            salt.utils.jid.LAST_JID_DATETIME = None
            ret = salt.utils.jid.gen_jid({'unique_jid': True})
            self.assertEqual(ret, '20021225120000000000_{0}'.format(os.getpid()))
            ret = salt.utils.jid.gen_jid({'unique_jid': True})
            self.assertEqual(ret, '20021225120000000001_{0}'.format(os.getpid()))



# -*- coding: utf-8 -*-
'''
Unit tests for the docker state
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    Mock,
    NO_MOCK,
    NO_MOCK_REASON,
    patch
)

# Import Salt Libs
import salt.modules.dockermod as docker_mod
import salt.states.docker_volume as docker_state


@skipIf(NO_MOCK, NO_MOCK_REASON)
class DockerVolumeTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test docker_volume states
    '''
    def setup_loader_modules(self):
        return {
            docker_mod: {
                '__context__': {'docker.docker_version': ''}
            },
            docker_state: {
                '__opts__': {'test': False}
            }
        }

    def test_present(self):
        '''
        Test docker_volume.present
        '''
        volumes = []
        default_driver = 'dummy_default'

        def create_volume(name, driver=None, driver_opts=None):
            for v in volumes:
                # present should never try to add a conflicting
                # volume
                self.assertNotEqual(v['Name'], name)
            if driver is None:
                driver = default_driver
            new = {'Name': name, 'Driver': driver}
            volumes.append(new)
            return new

        def remove_volume(name):
            old_len = len(volumes)
            removed = [v for v in volumes if v['Name'] == name]
            # present should not have tried to remove a volume
            # that didn't exist
            self.assertEqual(1, len(removed))
            volumes.remove(removed[0])
            return removed[0]

        docker_create_volume = Mock(side_effect=create_volume)
        __salt__ = {
            'docker.create_volume': docker_create_volume,
            'docker.volumes': Mock(return_value={'Volumes': volumes}),
            'docker.remove_volume': Mock(side_effect=remove_volume)
        }
        with patch.dict(docker_state.__dict__, {'__salt__': __salt__}):
            ret = docker_state.present('volume_foo')
            docker_create_volume.assert_called_with('volume_foo',
                                                    driver=None,
                                                    driver_opts=None)
            self.assertEqual(
                {
                    'name': 'volume_foo',
                    'comment': '',
                    'changes': {
                        'created': {
                            'Driver': default_driver,
                            'Name': 'volume_foo',
                        },
                    },
                    'result': True,
                },
                ret)
            self.assertEqual(len(volumes), 1)
            self.assertEqual(volumes[0]['Name'], 'volume_foo')
            self.assertIs(volumes[0]['Driver'], default_driver)

            # run it again with the same arguments
            orig_volumes = [volumes[0].copy()]
            ret = docker_state.present('volume_foo')
            self.assertEqual(
                {
                    'name': 'volume_foo',
                    'comment': "Volume 'volume_foo' already exists.",
                    'changes': {},
                    'result': True,
                },
                ret)
            self.assertEqual(orig_volumes, volumes)

            # run it again with a different driver but don't force
            ret = docker_state.present('volume_foo', driver='local')
            self.assertEqual(
                {
                    'name': 'volume_foo',
                    'comment': ("Driver for existing volume 'volume_foo'"
                                " ('dummy_default') does not match specified"
                                " driver ('local') and force is False"),
                    'changes': {},
                    'result': False,
                },
                ret)
            self.assertEqual(orig_volumes, volumes)

            # run it again with a different driver and force
            ret = docker_state.present(
                'volume_foo', driver='local', force=True)
            self.assertEqual(
                {
                    'name': 'volume_foo',
                    'comment': "",
                    'changes': {
                        'removed': {
                            'Driver': default_driver,
                            'Name': 'volume_foo',
                        },
                        'created': {
                            'Driver': 'local',
                            'Name': 'volume_foo',
                        },
                    },
                    'result': True,
                },
                ret)
            mod_orig_volumes = [orig_volumes[0].copy()]
            mod_orig_volumes[0]['Driver'] = 'local'
            self.assertEqual(mod_orig_volumes, volumes)

    def test_present_with_another_driver(self):
        '''
        Test docker_volume.present
        '''
        docker_create_volume = Mock(return_value='created')
        docker_remove_volume = Mock(return_value='removed')
        __salt__ = {'docker.create_volume': docker_create_volume,
                    'docker.remove_volume': docker_remove_volume,
                    'docker.volumes': Mock(return_value={
                        'Volumes': [{'Name': 'volume_foo',
                                     'Driver': 'foo'}]}),
                    }
        with patch.dict(docker_state.__dict__,
                        {'__salt__': __salt__}):
            ret = docker_state.present(
                'volume_foo',
                driver='bar',
                force=True,
                )
        docker_remove_volume.assert_called_with('volume_foo')
        docker_create_volume.assert_called_with('volume_foo',
                                                  driver='bar',
                                                  driver_opts=None)
        self.assertEqual(ret, {'name': 'volume_foo',
                               'comment': '',
                               'changes': {'created': 'created',
                                           'removed': 'removed'},
                               'result': True})

    def test_present_wo_existing_volumes(self):
        '''
        Test docker_volume.present without existing volumes.
        '''
        docker_create_volume = Mock(return_value='created')
        docker_remove_volume = Mock(return_value='removed')
        __salt__ = {'docker.create_volume': docker_create_volume,
                    'docker.remove_volume': docker_remove_volume,
                    'docker.volumes': Mock(return_value={'Volumes': None}),
                    }
        with patch.dict(docker_state.__dict__,
                        {'__salt__': __salt__}):
            ret = docker_state.present(
                'volume_foo',
                driver='bar',
                force=True,
                )
        docker_create_volume.assert_called_with('volume_foo',
                                                  driver='bar',
                                                  driver_opts=None)
        self.assertEqual(ret, {'name': 'volume_foo',
                               'comment': '',
                               'changes': {'created': 'created'},
                               'result': True})

    def test_absent(self):
        '''
        Test docker_volume.absent
        '''
        docker_remove_volume = Mock(return_value='removed')
        __salt__ = {'docker.remove_volume': docker_remove_volume,
                    'docker.volumes': Mock(return_value={
                        'Volumes': [{'Name': 'volume_foo'}]}),
                    }
        with patch.dict(docker_state.__dict__,
                        {'__salt__': __salt__}):
            ret = docker_state.absent(
                'volume_foo',
                )
        docker_remove_volume.assert_called_with('volume_foo')
        self.assertEqual(ret, {'name': 'volume_foo',
                               'comment': '',
                               'changes': {'removed': 'removed'},
                               'result': True})



# -*- coding: utf-8 -*-
'''
Simple Smoke Tests for Connected SSH minions
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.helpers import requires_sshd_server


@requires_sshd_server
class SSHMasterTestCase(ModuleCase):
    '''
    Test minion blackout functionality
    '''
    def test_can_it_ping(self):
        '''
        Ensure the proxy can ping
        '''
        ret = self.run_function('test.ping', minion_tgt='localhost')
        self.assertEqual(ret, True)

    def test_service(self):
        service = 'cron'
        os_family = self.run_function('grains.get', ['os_family'], minion_tgt='localhost')
        os_release = self.run_function('grains.get', ['osrelease'], minion_tgt='localhost')
        if os_family == 'RedHat':
            service = 'crond'
        elif os_family == 'Arch':
            service = 'sshd'
        elif os_family == 'MacOS':
            service = 'org.ntp.ntpd'
            if int(os_release.split('.')[1]) >= 13:
                service = 'com.apple.AirPlayXPCHelper'
        ret = self.run_function('service.get_all', minion_tgt='localhost')
        self.assertIn(service, ret)
        self.run_function('service.stop', [service], minion_tgt='localhost')
        ret = self.run_function('service.status', [service], minion_tgt='localhost')
        self.assertFalse(ret)
        self.run_function('service.start', [service], minion_tgt='localhost')
        ret = self.run_function('service.status', [service], minion_tgt='localhost')
        self.assertTrue(ret)

    def test_grains_items(self):
        os_family = self.run_function('grains.get', ['os_family'], minion_tgt='localhost')
        ret = self.run_function('grains.items', minion_tgt='localhost')
        if os_family == 'MacOS':
            self.assertEqual(ret['kernel'], 'Darwin')
        else:
            self.assertEqual(ret['kernel'], 'Linux')

    def test_state_apply(self):
        ret = self.run_function('state.apply', ['core'], minion_tgt='localhost')
        for key, value in ret.items():
            self.assertTrue(value['result'])

    def test_state_highstate(self):
        ret = self.run_function('state.highstate', minion_tgt='localhost')
        for key, value in ret.items():
            self.assertTrue(value['result'])



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.case import ModuleCase
from salt.ext import six


class SysModuleTest(ModuleCase):
    '''
    Validate the sys module
    '''
    def test_valid_docs(self):
        '''
        Make sure no functions are exposed that don't have valid docstrings
        '''
        ret = self.run_function('runtests_helpers.get_invalid_docs')
        if ret == {'missing_docstring': [], 'missing_cli_example': []}:
            return

        if isinstance(ret, six.string_types):
            self.fail(ret)

        self.fail(
            'There are some functions which do not have a docstring or do not '
            'have an example:\nNo docstring:\n{0}\nNo example:\n{1}\n'.format(
                '\n'.join(['  - {0}'.format(f) for f in ret['missing_docstring']]),
                '\n'.join(['  - {0}'.format(f) for f in ret['missing_cli_example']]),
            )
        )



# -*- coding: utf-8 -*-
'''
Unit tests for salt.utils.color.py
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.unit import TestCase

# Import Salt libs
import salt.utils.color
from salt.ext import six


class ColorUtilsTestCase(TestCase):

    def test_get_colors(self):
        ret = salt.utils.color.get_colors()
        self.assertEqual('\x1b[0;37m', six.text_type(ret['LIGHT_GRAY']))

        ret = salt.utils.color.get_colors(use=False)
        self.assertDictContainsSubset({'LIGHT_GRAY': ''}, ret)

        ret = salt.utils.color.get_colors(use='LIGHT_GRAY')
        # LIGHT_YELLOW now == LIGHT_GRAY
        self.assertEqual(six.text_type(ret['LIGHT_YELLOW']), six.text_type(ret['LIGHT_GRAY']))



# -*- coding: utf-8 -*-
'''
    tests.unit.utils.cache_test
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Test the salt cache objects
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import time
import tempfile
import shutil

# Import Salt Testing libs
from tests.support.unit import TestCase, skipIf
from tests.support.mock import NO_MOCK, NO_MOCK_REASON

# Import salt libs
import salt.config
import salt.loader
import salt.payload
import salt.utils.data
import salt.utils.files
import salt.utils.cache as cache


class CacheDictTestCase(TestCase):

    def test_sanity(self):
        '''
        Make sure you can instantiate etc.
        '''
        cd = cache.CacheDict(5)
        self.assertIsInstance(cd, cache.CacheDict)

        # do some tests to make sure it looks like a dict
        self.assertNotIn('foo', cd)
        cd['foo'] = 'bar'
        self.assertEqual(cd['foo'], 'bar')
        del cd['foo']
        self.assertNotIn('foo', cd)

    def test_ttl(self):
        cd = cache.CacheDict(0.1)
        cd['foo'] = 'bar'
        self.assertIn('foo', cd)
        self.assertEqual(cd['foo'], 'bar')
        time.sleep(0.2)
        self.assertNotIn('foo', cd)

        # make sure that a get would get a regular old key error
        self.assertRaises(KeyError, cd.__getitem__, 'foo')


class CacheContextTestCase(TestCase):

    def setUp(self):
        context_dir = os.path.join(tempfile.gettempdir(), 'context')
        if os.path.exists(context_dir):
            shutil.rmtree(os.path.join(tempfile.gettempdir(), 'context'))

    def test_smoke_context(self):
        '''
        Smoke test the context cache
        '''
        if os.path.exists(os.path.join(tempfile.gettempdir(), 'context')):
            self.skipTest('Context dir already exists')
        else:
            opts = salt.config.DEFAULT_MINION_OPTS
            opts['cachedir'] = tempfile.gettempdir()
            context_cache = cache.ContextCache(opts, 'cache_test')

            context_cache.cache_context({'a': 'b'})

            ret = context_cache.get_cache_context()

            self.assertDictEqual({'a': 'b'}, ret)

    def test_context_wrapper(self):
        '''
        Test to ensure that a module which decorates itself
        with a context cache can store and retrieve its contextual
        data
        '''
        opts = salt.config.DEFAULT_MINION_OPTS
        opts['cachedir'] = tempfile.gettempdir()

        ll_ = salt.loader.LazyLoader(
                [os.path.join(os.path.dirname(os.path.realpath(__file__)), 'cache_mods')],
                tag='rawmodule',
                virtual_enable=False,
                opts=opts)

        cache_test_func = ll_['cache_mod.test_context_module']

        self.assertEqual(cache_test_func()['called'], 0)
        self.assertEqual(cache_test_func()['called'], 1)


__context__ = {'a': 'b'}
__opts__ = {'cachedir': '/tmp'}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ContextCacheTest(TestCase):
    '''
    Test case for salt.utils.cache.ContextCache
    '''
    def setUp(self):
        '''
        Clear the cache before every test
        '''
        context_dir = os.path.join(__opts__['cachedir'], 'context')
        if os.path.isdir(context_dir):
            shutil.rmtree(context_dir)

    def test_set_cache(self):
        '''
        Tests to ensure the cache is written correctly
        '''
        @cache.context_cache
        def _test_set_cache():
            '''
            This will inherit globals from the test module itself.
            Normally these are injected by the salt loader [salt.loader]
            '''
            pass

        _test_set_cache()

        target_cache_file = os.path.join(__opts__['cachedir'], 'context', '{0}.p'.format(__name__))
        self.assertTrue(os.path.isfile(target_cache_file), 'Context cache did not write cache file')

        # Test manual de-serialize
        with salt.utils.files.fopen(target_cache_file, 'rb') as fp_:
            target_cache_data = salt.utils.data.decode(salt.payload.Serial(__opts__).load(fp_))
        self.assertDictEqual(__context__, target_cache_data)

        # Test cache de-serialize
        cc = cache.ContextCache(__opts__, __name__)
        retrieved_cache = cc.get_cache_context()
        self.assertDictEqual(retrieved_cache, __context__)

    def test_refill_cache(self):
        '''
        Tests to ensure that the context cache can rehydrate a wrapped function
        '''
        # First populate the cache
        @cache.context_cache
        def _test_set_cache():
            pass
        _test_set_cache()

        # Then try to rehydate a func
        @cache.context_cache
        def _test_refill_cache(comparison_context):
            self.assertEqual(__context__, comparison_context)

        global __context__
        __context__ = {}
        _test_refill_cache({'a': 'b'})  # Compare to the context before it was emptied


class CacheDiskTestCase(TestCase):

    def test_everything(self):
        '''
        Make sure you can instantiate, add, update, remove, expire
        '''
        try:
            tmpdir = tempfile.mkdtemp()
            path = os.path.join(tmpdir, 'CacheDisk_test')

            # test instantiation
            cd = cache.CacheDisk(0.1, path)
            self.assertIsInstance(cd, cache.CacheDisk)

            # test to make sure it looks like a dict
            self.assertNotIn('foo', cd)
            cd['foo'] = 'bar'
            self.assertIn('foo', cd)
            self.assertEqual(cd['foo'], 'bar')
            del cd['foo']
            self.assertNotIn('foo', cd)

            # test persistence
            cd['foo'] = 'bar'
            cd2 = cache.CacheDisk(0.1, path)
            self.assertIn('foo', cd2)
            self.assertEqual(cd2['foo'], 'bar')

            # test ttl
            time.sleep(0.2)
            self.assertNotIn('foo', cd)
            self.assertNotIn('foo', cd2)

        finally:
            shutil.rmtree(tmpdir, ignore_errors=True)



# -*- coding: utf-8 -*-

# Import Python Libs
from __future__ import absolute_import
import os

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.pkgin as pkgin


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PkginTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.pkgin
    '''
    def setup_loader_modules(self):
        return {
            pkgin: {
                '__opts__': {
                    'cachedir': '/tmp'
                }
            }
        }

    def test_search(self):
        '''
        Test searching for a package
        '''

        # Test searching for an available and uninstalled package
        pkgin_out = [
            'somepkg-1.0          Some package description here',
            '',
            '=: package is installed and up-to-date',
            '<: package is installed but newer version is available',
            '>: installed package has a greater version than available package'
        ]

        pkgin__get_version_mock = MagicMock(return_value=['0', '9', '0'])
        pkgin__check_pkgin_mock = MagicMock(return_value='/opt/pkg/bin/pkgin')
        pkgin_search_cmd = MagicMock(return_value=os.linesep.join(pkgin_out))

        with patch('salt.modules.pkgin._get_version', pkgin__get_version_mock), \
             patch('salt.modules.pkgin._check_pkgin', pkgin__check_pkgin_mock), \
             patch.dict(pkgin.__salt__, {'cmd.run': pkgin_search_cmd}):
            self.assertDictEqual(pkgin.search('somepkg'), {'somepkg': '1.0'})

        # Test searching for an available and installed package
        pkgin_out = [
            'somepkg-1.0 =          Some package description here',
            '',
            '=: package is installed and up-to-date',
            '<: package is installed but newer version is available',
            '>: installed package has a greater version than available package'
        ]

        pkgin_search_cmd = MagicMock(return_value=os.linesep.join(pkgin_out))

        with patch('salt.modules.pkgin._get_version', pkgin__get_version_mock), \
             patch('salt.modules.pkgin._check_pkgin', pkgin__check_pkgin_mock), \
             patch.dict(pkgin.__salt__, {'cmd.run': pkgin_search_cmd}):
            self.assertDictEqual(pkgin.search('somepkg'), {'somepkg': '1.0'})

    def test_latest_version(self):
        '''
        Test getting the latest version of a package
        '''

        # Test getting the latest version of an uninstalled package
        pkgin_out = [
            'somepkg-1.0;;Some package description here',
            '',
            '=: package is installed and up-to-date',
            '<: package is installed but newer version is available',
            '>: installed package has a greater version than available package'
        ]

        pkgin__get_version_mock = MagicMock(return_value=['0', '9', '0'])
        pkgin__check_pkgin_mock = MagicMock(return_value='/opt/pkg/bin/pkgin')
        pkgin_refresh_db_mock = MagicMock(return_value=True)
        pkgin_search_cmd = MagicMock(return_value=os.linesep.join(pkgin_out))

        with patch('salt.modules.pkgin.refresh_db', pkgin_refresh_db_mock), \
             patch('salt.modules.pkgin._get_version', pkgin__get_version_mock), \
             patch('salt.modules.pkgin._check_pkgin', pkgin__check_pkgin_mock), \
             patch.dict(pkgin.__salt__, {'cmd.run': pkgin_search_cmd}):
            self.assertEqual(pkgin.latest_version('somepkg'), '1.0')

        # Test getting the latest version of an installed package
        pkgin_out = [
            'somepkg-1.1;<;Some package description here',
            '',
            '=: package is installed and up-to-date',
            '<: package is installed but newer version is available',
            '>: installed package has a greater version than available package'
        ]

        pkgin_refresh_db_mock = MagicMock(return_value=True)
        pkgin_search_cmd = MagicMock(return_value=os.linesep.join(pkgin_out))

        with patch('salt.modules.pkgin.refresh_db', pkgin_refresh_db_mock), \
             patch('salt.modules.pkgin._get_version', pkgin__get_version_mock), \
             patch('salt.modules.pkgin._check_pkgin', pkgin__check_pkgin_mock), \
             patch.dict(pkgin.__salt__, {'cmd.run': pkgin_search_cmd}):
            self.assertEqual(pkgin.latest_version('somepkg'), '1.1')

        # Test getting the latest version of a package that is already installed
        # and is already at the latest version
        pkgin_out = [
            'somepkg-1.2;=;Some package description here',
            '',
            '=: package is installed and up-to-date',
            '<: package is installed but newer version is available',
            '>: installed package has a greater version than available package'
        ]

        pkgin_refresh_db_mock = MagicMock(return_value=True)
        pkgin_search_cmd = MagicMock(return_value=os.linesep.join(pkgin_out))

        with patch('salt.modules.pkgin.refresh_db', pkgin_refresh_db_mock), \
             patch('salt.modules.pkgin._get_version', pkgin__get_version_mock), \
             patch('salt.modules.pkgin._check_pkgin', pkgin__check_pkgin_mock), \
             patch.dict(pkgin.__salt__, {'cmd.run': pkgin_search_cmd}):
            self.assertEqual(pkgin.latest_version('somepkg'), '1.2')

        # Test getting the latest version of a bogus package
        pkgin_out = 'No results found for ^boguspkg$'

        pkgin_refresh_db_mock = MagicMock(return_value=True)
        pkgin_search_cmd = MagicMock(return_value=pkgin_out)

        with patch('salt.modules.pkgin.refresh_db', pkgin_refresh_db_mock), \
             patch('salt.modules.pkgin._get_version', pkgin__get_version_mock), \
             patch('salt.modules.pkgin._check_pkgin', pkgin__check_pkgin_mock), \
             patch.dict(pkgin.__salt__, {'cmd.run': pkgin_search_cmd}):
            self.assertEqual(pkgin.latest_version('boguspkg'), {})

    def test_file_dict(self):
        '''
        Test that file_dict doesn't crash
        '''
        pkg_info_stdout = [
            '/opt/pkg/bin/pkgin',
            '/opt/pkg/man/man1/pkgin.1',
            '/opt/pkg/share/examples/pkgin/preferred.conf.example',
            '/opt/pkg/share/examples/pkgin/repositories.conf.example'
        ]

        pkg_info_out = {
            'pid': 1234,
            'retcode': 0,
            'stderr': '',
            'stdout': os.linesep.join(pkg_info_stdout)
        }

        pkg_info_cmd = MagicMock(return_value=pkg_info_out)

        with patch.dict(pkgin.__salt__, {'cmd.run_all': pkg_info_cmd}):
            self.assertDictEqual(pkgin.file_dict('pkgin'), {
                'files': {
                'pkgin': [
                        '/opt/pkg/bin/pkgin',
                        '/opt/pkg/man/man1/pkgin.1',
                        '/opt/pkg/share/examples/pkgin/preferred.conf.example',
                        '/opt/pkg/share/examples/pkgin/repositories.conf.example'
                    ]
                }
            })



# -*- coding: utf-8 -*-
'''
Tests for salt.utils.jinja
'''
# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function
from jinja2 import Environment, DictLoader, exceptions
import ast
import copy
import datetime
import os
import pprint
import re
import tempfile

# Import Salt Testing libs
from tests.support.unit import skipIf, TestCase
from tests.support.case import ModuleCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch, MagicMock, Mock
from tests.support.paths import BASE_FILES, TMP, TMP_CONF_DIR

# Import Salt libs
import salt.config
import salt.loader
from salt.exceptions import SaltRenderError

from salt.ext import six
from salt.ext.six.moves import builtins

import salt.utils.json
from salt.utils.decorators.jinja import JinjaFilter
from salt.utils.jinja import (
    SaltCacheLoader,
    SerializerExtension,
    ensure_sequence_filter,
    tojson
)
from salt.utils.odict import OrderedDict
from salt.utils.templates import JINJA, render_jinja_tmpl

# dateutils is needed so that the strftime jinja filter is loaded
import salt.utils.dateutils  # pylint: disable=unused-import
import salt.utils.files
import salt.utils.stringutils
import salt.utils.yaml

# Import 3rd party libs
try:
    import timelib  # pylint: disable=W0611
    HAS_TIMELIB = True
except ImportError:
    HAS_TIMELIB = False

CACHEDIR = os.path.join(TMP, 'jinja-template-cache')
BLINESEP = salt.utils.stringutils.to_bytes(os.linesep)


class JinjaTestCase(TestCase):
    def test_tojson(self):
        '''
        Test the tojson filter for those using Jinja < 2.9. Non-ascii unicode
        content should be dumped with ensure_ascii=True.
        '''
        data = {'Non-ascii words': ['süß', 'спам', 'яйца']}
        result = tojson(data)
        expected = '{"Non-ascii words": ["s\\u00fc\\u00df", "\\u0441\\u043f\\u0430\\u043c", "\\u044f\\u0439\\u0446\\u0430"]}'
        assert result == expected, result


class MockFileClient(object):
    '''
    Does not download files but records any file request for testing
    '''
    def __init__(self, loader=None):
        if loader:
            loader._file_client = self
        self.requests = []

    def get_file(self, template, dest='', makedirs=False, saltenv='base'):
        self.requests.append({
            'path': template,
            'dest': dest,
            'makedirs': makedirs,
            'saltenv': saltenv
        })


def _setup_test_dir(src_dir, test_dir):
    os.makedirs(test_dir)
    salt.utils.files.recursive_copy(src_dir, test_dir)
    filename = os.path.join(test_dir, 'non_ascii')
    with salt.utils.files.fopen(filename, 'wb') as fp:
        fp.write(b'Assun\xc3\xa7\xc3\xa3o' + BLINESEP)
    filename = os.path.join(test_dir, 'hello_simple')
    with salt.utils.files.fopen(filename, 'wb') as fp:
        fp.write(b'world' + BLINESEP)
    filename = os.path.join(test_dir, 'hello_import')
    lines = [
        r"{% from 'macro' import mymacro -%}",
        r"{% from 'macro' import mymacro -%}",
        r"{{ mymacro('Hey') ~ mymacro(a|default('a'), b|default('b')) }}",
    ]
    with salt.utils.files.fopen(filename, 'wb') as fp:
        for line in lines:
            fp.write(line.encode('utf-8') + BLINESEP)


class TestSaltCacheLoader(TestCase):

    def setUp(self):
        self.tempdir = tempfile.mkdtemp()
        self.template_dir = os.path.join(self.tempdir, 'files', 'test')
        _setup_test_dir(
            os.path.join(BASE_FILES, 'templates'),
            self.template_dir
        )
        self.opts = {
            'cachedir': self.tempdir,
            'file_roots': {
                'test': [self.template_dir]
            },
            'pillar_roots': {
                'test': [self.template_dir]
            }
        }
        super(TestSaltCacheLoader, self).setUp()

    def tearDown(self):
        salt.utils.files.rm_rf(self.tempdir)

    def test_searchpath(self):
        '''
        The searchpath is based on the cachedir option and the saltenv parameter
        '''
        tmp = tempfile.gettempdir()
        opts = copy.deepcopy(self.opts)
        opts.update({'cachedir': tmp})
        loader = self.get_loader(opts=opts, saltenv='test')
        assert loader.searchpath == [os.path.join(tmp, 'files', 'test')]

    def test_mockclient(self):
        '''
        A MockFileClient is used that records all file requests normally sent
        to the master.
        '''
        loader = self.get_loader(opts=self.opts, saltenv='test')
        res = loader.get_source(None, 'hello_simple')
        assert len(res) == 3
        # res[0] on Windows is unicode and use os.linesep so it works cross OS
        self.assertEqual(six.text_type(res[0]), 'world' + os.linesep)
        tmpl_dir = os.path.join(self.template_dir, 'hello_simple')
        self.assertEqual(res[1], tmpl_dir)
        assert res[2](), 'Template up to date?'
        assert len(loader._file_client.requests)
        self.assertEqual(loader._file_client.requests[0]['path'], 'salt://hello_simple')

    def get_loader(self, opts=None, saltenv='base'):
        '''
        Now that we instantiate the client in the __init__, we need to mock it
        '''
        if opts is None:
            opts = self.opts
        with patch.object(SaltCacheLoader, 'file_client', Mock()):
            loader = SaltCacheLoader(opts, saltenv)
        # Create a mock file client and attach it to the loader
        MockFileClient(loader)
        return loader

    def get_test_saltenv(self):
        '''
        Setup a simple jinja test environment
        '''
        loader = self.get_loader(saltenv='test')
        jinja = Environment(loader=loader)
        return loader._file_client, jinja

    def test_import(self):
        '''
        You can import and use macros from other files
        '''
        fc, jinja = self.get_test_saltenv()
        result = jinja.get_template('hello_import').render()
        self.assertEqual(result, 'Hey world !a b !')
        assert len(fc.requests) == 2
        self.assertEqual(fc.requests[0]['path'], 'salt://hello_import')
        self.assertEqual(fc.requests[1]['path'], 'salt://macro')

    def test_include(self):
        '''
        You can also include a template that imports and uses macros
        '''
        fc, jinja = self.get_test_saltenv()
        result = jinja.get_template('hello_include').render()
        self.assertEqual(result, 'Hey world !a b !')
        assert len(fc.requests) == 3
        self.assertEqual(fc.requests[0]['path'], 'salt://hello_include')
        self.assertEqual(fc.requests[1]['path'], 'salt://hello_import')
        self.assertEqual(fc.requests[2]['path'], 'salt://macro')

    def test_include_context(self):
        '''
        Context variables are passes to the included template by default.
        '''
        _, jinja = self.get_test_saltenv()
        result = jinja.get_template('hello_include').render(a='Hi', b='Salt')
        self.assertEqual(result, 'Hey world !Hi Salt !')


class TestGetTemplate(TestCase):

    def setUp(self):
        self.tempdir = tempfile.mkdtemp()
        self.template_dir = os.path.join(self.tempdir, 'files', 'test')
        _setup_test_dir(
            os.path.join(BASE_FILES, 'templates'),
            self.template_dir
        )
        self.local_opts = {
            'cachedir': self.tempdir,
            'file_client': 'local',
            'file_ignore_regex': None,
            'file_ignore_glob': None,
            'file_roots': {
                'test': [self.template_dir]
            },
            'pillar_roots': {
                'test': [self.template_dir]
            },
            'fileserver_backend': ['roots'],
            'hash_type': 'md5',
            'extension_modules': os.path.join(
                os.path.dirname(os.path.abspath(__file__)),
                'extmods'),
        }
        self.local_salt = {}
        super(TestGetTemplate, self).setUp()

    def tearDown(self):
        salt.utils.files.rm_rf(self.tempdir)

    def test_fallback(self):
        '''
        A Template with a filesystem loader is returned as fallback
        if the file is not contained in the searchpath
        '''
        fn_ = os.path.join(self.template_dir, 'hello_simple')
        with salt.utils.files.fopen(fn_) as fp_:
            out = render_jinja_tmpl(
                salt.utils.stringutils.to_unicode(fp_.read()),
                dict(opts=self.local_opts, saltenv='test', salt=self.local_salt)
            )
        self.assertEqual(out, 'world' + os.linesep)

    def test_fallback_noloader(self):
        '''
        A Template with a filesystem loader is returned as fallback
        if the file is not contained in the searchpath
        '''
        filename = os.path.join(self.template_dir, 'hello_import')
        with salt.utils.files.fopen(filename) as fp_:
            out = render_jinja_tmpl(
                salt.utils.stringutils.to_unicode(fp_.read()),
                dict(opts=self.local_opts, saltenv='test', salt=self.local_salt)
            )
        self.assertEqual(out, 'Hey world !a b !' + os.linesep)

    def test_saltenv(self):
        '''
        If the template is within the searchpath it can
        import, include and extend other templates.
        The initial template is expected to be already cached
        get_template does not request it from the master again.
        '''
        fc = MockFileClient()
        with patch.object(SaltCacheLoader, 'file_client', MagicMock(return_value=fc)):
            filename = os.path.join(self.template_dir, 'hello_import')
            with salt.utils.files.fopen(filename) as fp_:
                out = render_jinja_tmpl(
                    salt.utils.stringutils.to_unicode(fp_.read()),
                    dict(opts={'cachedir': self.tempdir, 'file_client': 'remote',
                               'file_roots': self.local_opts['file_roots'],
                               'pillar_roots': self.local_opts['pillar_roots']},
                         a='Hi', b='Salt', saltenv='test', salt=self.local_salt))
            self.assertEqual(out, 'Hey world !Hi Salt !' + os.linesep)
            self.assertEqual(fc.requests[0]['path'], 'salt://macro')

    def test_macro_additional_log_for_generalexc(self):
        '''
        If we failed in a macro because of e.g. a TypeError, get
        more output from trace.
        '''
        expected = r'''Jinja error:.*division.*
.*macrogeneral\(2\):
---
\{% macro mymacro\(\) -%\}
\{\{ 1/0 \}\}    <======================
\{%- endmacro %\}
---.*'''
        filename = os.path.join(self.template_dir, 'hello_import_generalerror')
        fc = MockFileClient()
        with patch.object(SaltCacheLoader, 'file_client', MagicMock(return_value=fc)):
            with salt.utils.files.fopen(filename) as fp_:
                self.assertRaisesRegex(
                    SaltRenderError,
                    expected,
                    render_jinja_tmpl,
                    salt.utils.stringutils.to_unicode(fp_.read()),
                    dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))

    def test_macro_additional_log_for_undefined(self):
        '''
        If we failed in a macro because of undefined variables, get
        more output from trace.
        '''
        expected = r'''Jinja variable 'b' is undefined
.*macroundefined\(2\):
---
\{% macro mymacro\(\) -%\}
\{\{b.greetee\}\} <-- error is here    <======================
\{%- endmacro %\}
---'''
        filename = os.path.join(self.template_dir, 'hello_import_undefined')
        fc = MockFileClient()
        with patch.object(SaltCacheLoader, 'file_client', MagicMock(return_value=fc)):
            with salt.utils.files.fopen(filename) as fp_:
                self.assertRaisesRegex(
                    SaltRenderError,
                    expected,
                    render_jinja_tmpl,
                    salt.utils.stringutils.to_unicode(fp_.read()),
                    dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))

    def test_macro_additional_log_syntaxerror(self):
        '''
        If  we failed in a macro, get more output from trace.
        '''
        expected = r'''Jinja syntax error: expected token .*end.*got '-'.*
.*macroerror\(2\):
---
# macro
\{% macro mymacro\(greeting, greetee='world'\) -\} <-- error is here    <======================
\{\{ greeting ~ ' ' ~ greetee \}\} !
\{%- endmacro %\}
---.*'''
        filename = os.path.join(self.template_dir, 'hello_import_error')
        fc = MockFileClient()
        with patch.object(SaltCacheLoader, 'file_client', MagicMock(return_value=fc)):
            with salt.utils.files.fopen(filename) as fp_:
                self.assertRaisesRegex(
                    SaltRenderError,
                    expected,
                    render_jinja_tmpl,
                    salt.utils.stringutils.to_unicode(fp_.read()),
                    dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))

    def test_non_ascii_encoding(self):
        fc = MockFileClient()
        with patch.object(SaltCacheLoader, 'file_client', MagicMock(return_value=fc)):
            filename = os.path.join(self.template_dir, 'hello_import')
            with salt.utils.files.fopen(filename) as fp_:
                out = render_jinja_tmpl(
                    salt.utils.stringutils.to_unicode(fp_.read()),
                    dict(opts={'cachedir': self.tempdir, 'file_client': 'remote',
                               'file_roots': self.local_opts['file_roots'],
                               'pillar_roots': self.local_opts['pillar_roots']},
                         a='Hi', b='Sàlt', saltenv='test', salt=self.local_salt))
            self.assertEqual(out, salt.utils.stringutils.to_unicode('Hey world !Hi Sàlt !' + os.linesep))
            self.assertEqual(fc.requests[0]['path'], 'salt://macro')

            filename = os.path.join(self.template_dir, 'non_ascii')
            with salt.utils.files.fopen(filename, 'rb') as fp_:
                out = render_jinja_tmpl(
                    salt.utils.stringutils.to_unicode(fp_.read(), 'utf-8'),
                    dict(opts={'cachedir': self.tempdir, 'file_client': 'remote',
                               'file_roots': self.local_opts['file_roots'],
                               'pillar_roots': self.local_opts['pillar_roots']},
                         a='Hi', b='Sàlt', saltenv='test', salt=self.local_salt))
            self.assertEqual('Assunção' + os.linesep, out)
            self.assertEqual(fc.requests[0]['path'], 'salt://macro')

    @skipIf(HAS_TIMELIB is False, 'The `timelib` library is not installed.')
    def test_strftime(self):
        response = render_jinja_tmpl(
            '{{ "2002/12/25"|strftime }}',
            dict(
                opts=self.local_opts,
                saltenv='test',
                salt=self.local_salt
            ))
        self.assertEqual(response, '2002-12-25')

        objects = (
            datetime.datetime(2002, 12, 25, 12, 00, 00, 00),
            '2002/12/25',
            1040814000,
            '1040814000'
        )

        for object in objects:
            response = render_jinja_tmpl(
                '{{ object|strftime }}',
                dict(
                    object=object,
                    opts=self.local_opts,
                    saltenv='test',
                    salt=self.local_salt
                ))
            self.assertEqual(response, '2002-12-25')

            response = render_jinja_tmpl(
                '{{ object|strftime("%b %d, %Y") }}',
                dict(
                    object=object,
                    opts=self.local_opts,
                    saltenv='test',
                    salt=self.local_salt
                ))
            self.assertEqual(response, 'Dec 25, 2002')

            response = render_jinja_tmpl(
                '{{ object|strftime("%y") }}',
                dict(
                    object=object,
                    opts=self.local_opts,
                    saltenv='test',
                    salt=self.local_salt
                ))
            self.assertEqual(response, '02')

    def test_non_ascii(self):
        fn = os.path.join(self.template_dir, 'non_ascii')
        out = JINJA(
            fn,
            opts=self.local_opts,
            saltenv='test',
            salt=self.local_salt
        )
        with salt.utils.files.fopen(out['data'], 'rb') as fp:
            result = salt.utils.stringutils.to_unicode(fp.read(), 'utf-8')
            self.assertEqual(salt.utils.stringutils.to_unicode('Assunção' + os.linesep), result)

    def test_get_context_has_enough_context(self):
        template = '1\n2\n3\n4\n5\n6\n7\n8\n9\na\nb\nc\nd\ne\nf'
        context = salt.utils.stringutils.get_context(template, 8)
        expected = '---\n[...]\n3\n4\n5\n6\n7\n8\n9\na\nb\nc\nd\n[...]\n---'
        self.assertEqual(expected, context)

    def test_get_context_at_top_of_file(self):
        template = '1\n2\n3\n4\n5\n6\n7\n8\n9\na\nb\nc\nd\ne\nf'
        context = salt.utils.stringutils.get_context(template, 1)
        expected = '---\n1\n2\n3\n4\n5\n6\n[...]\n---'
        self.assertEqual(expected, context)

    def test_get_context_at_bottom_of_file(self):
        template = '1\n2\n3\n4\n5\n6\n7\n8\n9\na\nb\nc\nd\ne\nf'
        context = salt.utils.stringutils.get_context(template, 15)
        expected = '---\n[...]\na\nb\nc\nd\ne\nf\n---'
        self.assertEqual(expected, context)

    def test_get_context_2_context_lines(self):
        template = '1\n2\n3\n4\n5\n6\n7\n8\n9\na\nb\nc\nd\ne\nf'
        context = salt.utils.stringutils.get_context(template, 8, num_lines=2)
        expected = '---\n[...]\n6\n7\n8\n9\na\n[...]\n---'
        self.assertEqual(expected, context)

    def test_get_context_with_marker(self):
        template = '1\n2\n3\n4\n5\n6\n7\n8\n9\na\nb\nc\nd\ne\nf'
        context = salt.utils.stringutils.get_context(template, 8, num_lines=2, marker=' <---')
        expected = '---\n[...]\n6\n7\n8 <---\n9\na\n[...]\n---'
        self.assertEqual(expected, context)

    def test_render_with_syntax_error(self):
        template = 'hello\n\n{{ bad\n\nfoo'
        expected = r'.*---\nhello\n\n{{ bad\n\nfoo    <======================\n---'
        self.assertRaisesRegex(
            SaltRenderError,
            expected,
            render_jinja_tmpl,
            template,
            dict(opts=self.local_opts, saltenv='test', salt=self.local_salt)
        )

    @skipIf(six.PY3, 'Not applicable to Python 3')
    @skipIf(NO_MOCK, NO_MOCK_REASON)
    def test_render_with_unicode_syntax_error(self):
        with patch.object(builtins, '__salt_system_encoding__', 'utf-8'):
            template = 'hello\n\n{{ bad\n\nfoo한'
            expected = r'.*---\nhello\n\n{{ bad\n\nfoo\xed\x95\x9c    <======================\n---'
            self.assertRaisesRegex(
                SaltRenderError,
                expected,
                render_jinja_tmpl,
                template,
                dict(opts=self.local_opts, saltenv='test', salt=self.local_salt)
            )

    @skipIf(NO_MOCK, NO_MOCK_REASON)
    def test_render_with_utf8_syntax_error(self):
        with patch.object(builtins, '__salt_system_encoding__', 'utf-8'):
            template = 'hello\n\n{{ bad\n\nfoo한'
            expected = salt.utils.stringutils.to_str(
                r'.*---\nhello\n\n{{ bad\n\nfoo한    <======================\n---'
            )
            self.assertRaisesRegex(
                SaltRenderError,
                expected,
                render_jinja_tmpl,
                template,
                dict(opts=self.local_opts, saltenv='test', salt=self.local_salt)
            )

    def test_render_with_undefined_variable(self):
        template = "hello\n\n{{ foo }}\n\nfoo"
        expected = r'Jinja variable \'foo\' is undefined'
        self.assertRaisesRegex(
            SaltRenderError,
            expected,
            render_jinja_tmpl,
            template,
            dict(opts=self.local_opts, saltenv='test', salt=self.local_salt)
        )

    def test_render_with_undefined_variable_utf8(self):
        template = "hello\xed\x95\x9c\n\n{{ foo }}\n\nfoo"
        expected = r'Jinja variable \'foo\' is undefined'
        self.assertRaisesRegex(
            SaltRenderError,
            expected,
            render_jinja_tmpl,
            template,
            dict(opts=self.local_opts, saltenv='test', salt=self.local_salt)
        )

    def test_render_with_undefined_variable_unicode(self):
        template = 'hello한\n\n{{ foo }}\n\nfoo'
        expected = r'Jinja variable \'foo\' is undefined'
        self.assertRaisesRegex(
            SaltRenderError,
            expected,
            render_jinja_tmpl,
            template,
            dict(opts=self.local_opts, saltenv='test', salt=self.local_salt)
        )


class TestJinjaDefaultOptions(TestCase):

    def __init__(self, *args, **kws):
        TestCase.__init__(self, *args, **kws)
        self.local_opts = {
            'cachedir': CACHEDIR,
            'file_client': 'local',
            'file_ignore_regex': None,
            'file_ignore_glob': None,
            'file_roots': {
                'test': [os.path.join(BASE_FILES, 'templates')]
            },
            'pillar_roots': {
                'test': [os.path.join(BASE_FILES, 'templates')]
            },
            'fileserver_backend': ['roots'],
            'hash_type': 'md5',
            'extension_modules': os.path.join(
                os.path.dirname(os.path.abspath(__file__)),
                'extmods'),
            'jinja_env': {
                'line_comment_prefix': '##',
                'line_statement_prefix': '%',
            },
        }
        self.local_salt = {
             'myvar': 'zero',
             'mylist': [0, 1, 2, 3],
        }

    def test_comment_prefix(self):

        template = """
            %- set myvar = 'one'
            ## ignored comment 1
            {{- myvar -}}
            {%- set myvar = 'two' %} ## ignored comment 2
            {{- myvar }} ## ignored comment 3
            %- if myvar == 'two':
            %- set myvar = 'three'
            %- endif
            {{- myvar -}}
            """
        rendered = render_jinja_tmpl(template,
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'onetwothree')

    def test_statement_prefix(self):

        template = """
            {%- set mylist = ['1', '2', '3'] %}
            %- set mylist = ['one', 'two', 'three']
            %- for item in mylist:
            {{- item }}
            %- endfor
            """
        rendered = render_jinja_tmpl(template,
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'onetwothree')


class TestCustomExtensions(TestCase):

    def __init__(self, *args, **kws):
        super(TestCustomExtensions, self).__init__(*args, **kws)
        self.local_opts = {
            'cachedir': CACHEDIR,
            'file_client': 'local',
            'file_ignore_regex': None,
            'file_ignore_glob': None,
            'file_roots': {
                'test': [os.path.join(BASE_FILES, 'templates')]
            },
            'pillar_roots': {
                'test': [os.path.join(BASE_FILES, 'templates')]
            },
            'fileserver_backend': ['roots'],
            'hash_type': 'md5',
            'extension_modules': os.path.join(
                os.path.dirname(os.path.abspath(__file__)),
                'extmods'),
        }
        self.local_salt = {
            # 'dns.A': dnsutil.A,
            # 'dns.AAAA': dnsutil.AAAA,
            # 'file.exists': filemod.file_exists,
            # 'file.basename': filemod.basename,
            # 'file.dirname': filemod.dirname
        }

    def test_regex_escape(self):
        dataset = 'foo?:.*/\\bar'
        env = Environment(extensions=[SerializerExtension])
        env.filters.update(JinjaFilter.salt_jinja_filters)
        rendered = env.from_string('{{ dataset|regex_escape }}').render(dataset=dataset)
        self.assertEqual(rendered, re.escape(dataset))

    def test_unique_string(self):
        dataset = 'foo'
        unique = set(dataset)
        env = Environment(extensions=[SerializerExtension])
        env.filters.update(JinjaFilter.salt_jinja_filters)
        if six.PY3:
            rendered = env.from_string('{{ dataset|unique }}').render(dataset=dataset).strip("'{}").split("', '")
            self.assertEqual(sorted(rendered), sorted(list(unique)))
        else:
            rendered = env.from_string('{{ dataset|unique }}').render(dataset=dataset)
            self.assertEqual(rendered, "{0}".format(unique))

    def test_unique_tuple(self):
        dataset = ('foo', 'foo', 'bar')
        unique = set(dataset)
        env = Environment(extensions=[SerializerExtension])
        env.filters.update(JinjaFilter.salt_jinja_filters)
        if six.PY3:
            rendered = env.from_string('{{ dataset|unique }}').render(dataset=dataset).strip("'{}").split("', '")
            self.assertEqual(sorted(rendered), sorted(list(unique)))
        else:
            rendered = env.from_string('{{ dataset|unique }}').render(dataset=dataset)
            self.assertEqual(rendered, "{0}".format(unique))

    def test_unique_list(self):
        dataset = ['foo', 'foo', 'bar']
        unique = ['foo', 'bar']
        env = Environment(extensions=[SerializerExtension])
        env.filters.update(JinjaFilter.salt_jinja_filters)
        if six.PY3:
            rendered = env.from_string('{{ dataset|unique }}').render(dataset=dataset).strip("'[]").split("', '")
            self.assertEqual(rendered, unique)
        else:
            rendered = env.from_string('{{ dataset|unique }}').render(dataset=dataset)
            self.assertEqual(rendered, "{0}".format(unique))

    def test_serialize_json(self):
        dataset = {
            "foo": True,
            "bar": 42,
            "baz": [1, 2, 3],
            "qux": 2.0
        }
        env = Environment(extensions=[SerializerExtension])
        rendered = env.from_string('{{ dataset|json }}').render(dataset=dataset)
        self.assertEqual(dataset, salt.utils.json.loads(rendered))

    def test_serialize_yaml(self):
        dataset = {
            "foo": True,
            "bar": 42,
            "baz": [1, 2, 3],
            "qux": 2.0,
            "spam": OrderedDict([
                ('foo', OrderedDict([
                    ('bar', 'baz'),
                    ('qux', 42)
                ])
                )
            ])
        }
        env = Environment(extensions=[SerializerExtension])
        rendered = env.from_string('{{ dataset|yaml }}').render(dataset=dataset)
        self.assertEqual(dataset, salt.utils.yaml.safe_load(rendered))

    def test_serialize_yaml_str(self):
        dataset = "str value"
        env = Environment(extensions=[SerializerExtension])
        rendered = env.from_string('{{ dataset|yaml }}').render(dataset=dataset)
        self.assertEqual(dataset, rendered)

    def test_serialize_yaml_unicode(self):
        dataset = 'str value'
        env = Environment(extensions=[SerializerExtension])
        rendered = env.from_string('{{ dataset|yaml }}').render(dataset=dataset)
        if six.PY3:
            self.assertEqual("str value", rendered)
        else:
            # Due to a bug in the equality handler, this check needs to be split
            # up into several different assertions. We need to check that the various
            # string segments are present in the rendered value, as well as the
            # type of the rendered variable (should be unicode, which is the same as
            # six.text_type). This should cover all use cases but also allow the test
            # to pass on CentOS 6 running Python 2.7.
            self.assertIn('str value', rendered)
            self.assertIsInstance(rendered, six.text_type)

    def test_serialize_python(self):
        dataset = {
            "foo": True,
            "bar": 42,
            "baz": [1, 2, 3],
            "qux": 2.0
        }
        env = Environment(extensions=[SerializerExtension])
        rendered = env.from_string('{{ dataset|python }}').render(dataset=dataset)
        self.assertEqual(rendered, pprint.pformat(dataset))

    def test_load_yaml(self):
        env = Environment(extensions=[SerializerExtension])
        rendered = env.from_string('{% set document = "{foo: it works}"|load_yaml %}{{ document.foo }}').render()
        self.assertEqual(rendered, "it works")

        rendered = env.from_string('{% set document = document|load_yaml %}'
                                   '{{ document.foo }}').render(document="{foo: it works}")
        self.assertEqual(rendered, "it works")

        with self.assertRaises(exceptions.TemplateRuntimeError):
            env.from_string('{% set document = document|load_yaml %}'
                                       '{{ document.foo }}').render(document={"foo": "it works"})

    def test_load_tag(self):
        env = Environment(extensions=[SerializerExtension])

        source = '{{ bar }}, ' + \
                 '{% load_yaml as docu %}{foo: it works, {{ bar }}: baz}{% endload %}' + \
                                        '{{ docu.foo }}'

        rendered = env.from_string(source).render(bar="barred")
        self.assertEqual(rendered, "barred, it works")

        source = '{{ bar }}, {% load_json as docu %}{"foo": "it works", "{{ bar }}": "baz"}{% endload %}' + \
                                        '{{ docu.foo }}'

        rendered = env.from_string(source).render(bar="barred")
        self.assertEqual(rendered, "barred, it works")

        with self.assertRaises(exceptions.TemplateSyntaxError):
            env.from_string('{% load_yamle as document %}{foo, bar: it works}{% endload %}').render()

        with self.assertRaises(exceptions.TemplateRuntimeError):
            env.from_string('{% load_json as document %}{foo, bar: it works}{% endload %}').render()

    def test_load_json(self):
        env = Environment(extensions=[SerializerExtension])
        rendered = env.from_string('{% set document = \'{"foo": "it works"}\'|load_json %}'
                                   '{{ document.foo }}').render()
        self.assertEqual(rendered, "it works")

        rendered = env.from_string('{% set document = document|load_json %}'
                                   '{{ document.foo }}').render(document='{"foo": "it works"}')
        self.assertEqual(rendered, "it works")

        # bad quotes
        with self.assertRaises(exceptions.TemplateRuntimeError):
            env.from_string("{{ document|load_json }}").render(document="{'foo': 'it works'}")

        # not a string
        with self.assertRaises(exceptions.TemplateRuntimeError):
            env.from_string('{{ document|load_json }}').render(document={"foo": "it works"})

    def test_load_yaml_template(self):
        loader = DictLoader({'foo': '{bar: "my god is blue", foo: [1, 2, 3]}'})
        env = Environment(extensions=[SerializerExtension], loader=loader)
        rendered = env.from_string('{% import_yaml "foo" as doc %}{{ doc.bar }}').render()
        self.assertEqual(rendered, "my god is blue")

        with self.assertRaises(exceptions.TemplateNotFound):
            env.from_string('{% import_yaml "does not exists" as doc %}').render()

    def test_load_json_template(self):
        loader = DictLoader({'foo': '{"bar": "my god is blue", "foo": [1, 2, 3]}'})
        env = Environment(extensions=[SerializerExtension], loader=loader)
        rendered = env.from_string('{% import_json "foo" as doc %}{{ doc.bar }}').render()
        self.assertEqual(rendered, "my god is blue")

        with self.assertRaises(exceptions.TemplateNotFound):
            env.from_string('{% import_json "does not exists" as doc %}').render()

    def test_load_text_template(self):
        loader = DictLoader({'foo': 'Foo!'})
        env = Environment(extensions=[SerializerExtension], loader=loader)

        rendered = env.from_string('{% import_text "foo" as doc %}{{ doc }}').render()
        self.assertEqual(rendered, "Foo!")

        with self.assertRaises(exceptions.TemplateNotFound):
            env.from_string('{% import_text "does not exists" as doc %}').render()

    def test_catalog(self):
        loader = DictLoader({
            'doc1': '{bar: "my god is blue"}',
            'doc2': '{% import_yaml "doc1" as local2 %} never exported',
            'doc3': '{% load_yaml as local3 %}{"foo": "it works"}{% endload %} me neither',
            'main1': '{% from "doc2" import local2 %}{{ local2.bar }}',
            'main2': '{% from "doc3" import local3 %}{{ local3.foo }}',
            'main3': '''
                {% import "doc2" as imported2 %}
                {% import "doc3" as imported3 %}
                {{ imported2.local2.bar }}
            ''',
            'main4': '''
                {% import "doc2" as imported2 %}
                {% import "doc3" as imported3 %}
                {{ imported3.local3.foo }}
            ''',
            'main5': '''
                {% from "doc2" import local2 as imported2 %}
                {% from "doc3" import local3 as imported3 %}
                {{ imported2.bar }}
            ''',
            'main6': '''
                {% from "doc2" import local2 as imported2 %}
                {% from "doc3" import local3 as imported3 %}
                {{ imported3.foo }}
            '''

        })

        env = Environment(extensions=[SerializerExtension], loader=loader)
        rendered = env.get_template('main1').render()
        self.assertEqual(rendered, "my god is blue")

        rendered = env.get_template('main2').render()
        self.assertEqual(rendered, "it works")

        rendered = env.get_template('main3').render().strip()
        self.assertEqual(rendered, "my god is blue")

        rendered = env.get_template('main4').render().strip()
        self.assertEqual(rendered, "it works")

        rendered = env.get_template('main5').render().strip()
        self.assertEqual(rendered, "my god is blue")

        rendered = env.get_template('main6').render().strip()
        self.assertEqual(rendered, "it works")

    def test_nested_structures(self):
        env = Environment(extensions=[SerializerExtension])
        rendered = env.from_string('{{ data }}').render(data="foo")
        self.assertEqual(rendered, "foo")

        data = OrderedDict([
            ('foo', OrderedDict([
                        ('bar', 'baz'),
                        ('qux', 42)
                    ])
            )
        ])

        rendered = env.from_string('{{ data }}').render(data=data)
        self.assertEqual(
            rendered,
            "{u'foo': {u'bar': u'baz', u'qux': 42}}" if six.PY2
                else "{'foo': {'bar': 'baz', 'qux': 42}}"
        )

        rendered = env.from_string('{{ data }}').render(data=[
                                                            OrderedDict(
                                                                foo='bar',
                                                            ),
                                                            OrderedDict(
                                                                baz=42,
                                                            )
                                                        ])
        self.assertEqual(
            rendered,
            "[{'foo': u'bar'}, {'baz': 42}]" if six.PY2
                else "[{'foo': 'bar'}, {'baz': 42}]"
        )

    def test_sequence(self):
        env = Environment()
        env.filters['sequence'] = ensure_sequence_filter

        rendered = env.from_string('{{ data | sequence | length }}') \
                      .render(data='foo')
        self.assertEqual(rendered, '1')

        rendered = env.from_string('{{ data | sequence | length }}') \
                      .render(data=['foo', 'bar'])
        self.assertEqual(rendered, '2')

        rendered = env.from_string('{{ data | sequence | length }}') \
                      .render(data=('foo', 'bar'))
        self.assertEqual(rendered, '2')

        rendered = env.from_string('{{ data | sequence | length }}') \
                      .render(data=set(['foo', 'bar']))
        self.assertEqual(rendered, '2')

        rendered = env.from_string('{{ data | sequence | length }}') \
                      .render(data={'foo': 'bar'})
        self.assertEqual(rendered, '1')

    def test_is_ip(self):
        '''
        Test the `is_ip` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ '192.168.0.1' | is_ip }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'True')

        rendered = render_jinja_tmpl("{{ 'FE80::' | is_ip }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'True')

        rendered = render_jinja_tmpl("{{ 'random' | is_ip }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'False')

    def test_is_ipv4(self):
        '''
        Test the `is_ipv4` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ '192.168.0.1' | is_ipv4 }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'True')

        rendered = render_jinja_tmpl("{{ 'FE80::' | is_ipv4 }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'False')

        rendered = render_jinja_tmpl("{{ 'random' | is_ipv4 }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'False')

    def test_is_ipv6(self):
        '''
        Test the `is_ipv6` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ '192.168.0.1' | is_ipv6 }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'False')

        rendered = render_jinja_tmpl("{{ 'fe80::20d:b9ff:fe01:ea8%eth0' | is_ipv6 }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'True')

        rendered = render_jinja_tmpl("{{ 'FE80::' | is_ipv6 }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'True')

        rendered = render_jinja_tmpl("{{ 'random' | is_ipv6 }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'False')

    def test_ipaddr(self):
        '''
        Test the `ipaddr` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ '::' | ipaddr }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '::')

        rendered = render_jinja_tmpl("{{ '192.168.0.1' | ipaddr }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '192.168.0.1')

        # provides a list with valid IP addresses only
        rendered = render_jinja_tmpl("{{ ['192.168.0.1', '172.17.17.1', 'foo', 'bar', '::'] | ipaddr | join(', ') }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '192.168.0.1, 172.17.17.1, ::')

        # return only multicast addresses
        rendered = render_jinja_tmpl("{{ ['224.0.0.1', 'FF01::1', '::'] | ipaddr(options='multicast') | join(', ') }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '224.0.0.1, ff01::1')

    def test_ipv4(self):
        '''
        Test the `ipv4` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ '192.168.0.1' | ipv4 }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '192.168.0.1')

        rendered = render_jinja_tmpl("{{ ['192.168.0.1', '172.17.17.1'] | ipv4 | join(', ')}}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '192.168.0.1, 172.17.17.1')

        rendered = render_jinja_tmpl("{{ 'fe80::' | ipv4 }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'None')

        rendered = render_jinja_tmpl("{{ 'random' | ipv4 }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'None')

        rendered = render_jinja_tmpl("{{ '192.168.0.1' | ipv4(options='lo') }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'None')

        rendered = render_jinja_tmpl("{{ '127.0.0.1' | ipv4(options='lo') }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '127.0.0.1')

    def test_ipv6(self):
        '''
        Test the `ipv6` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ '192.168.0.1' | ipv6 }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'None')

        rendered = render_jinja_tmpl("{{ 'random' | ipv6 }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'None')

        # returns the standard format value
        rendered = render_jinja_tmpl("{{ 'FE80:0:0::0' | ipv6 }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'fe80::')

        # fe80:: is link local therefore will be returned
        rendered = render_jinja_tmpl("{{ 'fe80::' | ipv6(options='ll') }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'fe80::')

        # fe80:: is not loopback
        rendered = render_jinja_tmpl("{{ 'fe80::' | ipv6(options='lo') }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'None')

        # returns only IPv6 addresses in the list
        rendered = render_jinja_tmpl("{{ ['fe80::', '192.168.0.1'] | ipv6 | join(', ') }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'fe80::')

        rendered = render_jinja_tmpl("{{ ['fe80::', '::'] | ipv6 | join(', ') }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'fe80::, ::')

    def test_network_hosts(self):
        '''
        Test the `network_hosts` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ '192.168.0.1/30' | network_hosts | join(', ') }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '192.168.0.1, 192.168.0.2')

    def test_network_size(self):
        '''
        Test the `network_size` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ '192.168.0.1' | network_size }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '1')

        rendered = render_jinja_tmpl("{{ '192.168.0.1/8' | network_size }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '16777216')

    def test_http_query(self):
        '''
        Test the `http_query` Jinja filter.
        '''
        for backend in ('requests', 'tornado', 'urllib2'):
            rendered = render_jinja_tmpl("{{ 'http://icanhazip.com' | http_query(backend='" + backend + "') }}",
                                         dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
            self.assertIsInstance(rendered, six.text_type, 'Failed with backend: {}'.format(backend))
            dict_reply = ast.literal_eval(rendered)
            self.assertIsInstance(dict_reply, dict, 'Failed with backend: {}'.format(backend))
            self.assertIsInstance(dict_reply['body'], six.string_types, 'Failed with backend: {}'.format(backend))

    def test_to_bool(self):
        '''
        Test the `to_bool` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ 1 | to_bool }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'True')

        rendered = render_jinja_tmpl("{{ 'True' | to_bool }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'True')

        rendered = render_jinja_tmpl("{{ 0 | to_bool }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'False')

        rendered = render_jinja_tmpl("{{ 'Yes' | to_bool }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'True')

    def test_quote(self):
        '''
        Test the `quote` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ 'random' | quote }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'random')

    def test_regex_search(self):
        '''
        Test the `regex_search` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ 'abcdefabcdef' | regex_search('BC(.*)', ignorecase=True) }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, "('defabcdef',)")  # because search looks only at the beginning

    def test_regex_match(self):
        '''
        Test the `regex_match` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ 'abcdefabcdef' | regex_match('BC(.*)', ignorecase=True)}}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, "None")

    def test_regex_replace(self):
        '''
        Test the `regex_replace` Jinja filter.
        '''
        rendered = render_jinja_tmpl(r"{{ 'lets replace spaces' | regex_replace('\s+', '__') }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'lets__replace__spaces')

    def test_uuid(self):
        '''
        Test the `uuid` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ 'random' | uuid }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '3652b285-26ad-588e-a5dc-c2ee65edc804')

    def test_min(self):
        '''
        Test the `min` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ [1, 2, 3] | min }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '1')

    def test_max(self):
        '''
        Test the `max` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ [1, 2, 3] | max }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '3')

    def test_avg(self):
        '''
        Test the `avg` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ [1, 2, 3] | avg }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '2.0')

    def test_union(self):
        '''
        Test the `union` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ [1, 2, 3] | union([2, 3, 4]) | join(', ') }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '1, 2, 3, 4')

    def test_intersect(self):
        '''
        Test the `intersect` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ [1, 2, 3] | intersect([2, 3, 4]) | join(', ') }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '2, 3')

    def test_difference(self):
        '''
        Test the `difference` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ [1, 2, 3] | difference([2, 3, 4]) | join(', ') }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '1')

    def test_symmetric_difference(self):
        '''
        Test the `symmetric_difference` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ [1, 2, 3] | symmetric_difference([2, 3, 4]) | join(', ') }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '1, 4')

    def test_md5(self):
        '''
        Test the `md5` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ 'random' | md5 }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, '7ddf32e17a6ac5ce04a8ecbf782ca509')

    def test_sha256(self):
        '''
        Test the `sha256` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ 'random' | sha256 }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'a441b15fe9a3cf56661190a0b93b9dec7d04127288cc87250967cf3b52894d11')

    def test_sha512(self):
        '''
        Test the `sha512` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ 'random' | sha512 }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, six.text_type(('811a90e1c8e86c7b4c0eef5b2c0bf0ec1b19c4b1b5a242e6455be93787cb473cb7bc'
                                                  '9b0fdeb960d00d5c6881c2094dd63c5c900ce9057255e2a4e271fc25fef1')))

    def test_hmac(self):
        '''
        Test the `hmac` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ 'random' | hmac('secret', 'blah') }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'False')

        rendered = render_jinja_tmpl(("{{ 'get salted' | "
                                      "hmac('shared secret', 'eBWf9bstXg+NiP5AOwppB5HMvZiYMPzEM9W5YMm/AmQ=') }}"),
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'True')

    def test_base64_encode(self):
        '''
        Test the `base64_encode` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ 'random' | base64_encode }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'cmFuZG9t')

    def test_base64_decode(self):
        '''
        Test the `base64_decode` Jinja filter.
        '''
        rendered = render_jinja_tmpl("{{ 'cmFuZG9t' | base64_decode }}",
                                     dict(opts=self.local_opts, saltenv='test', salt=self.local_salt))
        self.assertEqual(rendered, 'random')

    # def test_print(self):
    #     env = Environment(extensions=[SerializerExtension])
    #     source = '{% import_yaml "toto.foo" as docu %}'
    #     name, filename = None, '<filename>'
    #     parsed = env._parse(source, name, filename)
    #     print parsed
    #     print
    #     compiled = env._generate(parsed, name, filename)
    #     print compiled
    #     return


class TestDotNotationLookup(ModuleCase):
    '''
    Tests to call Salt functions via Jinja with various lookup syntaxes
    '''
    def setUp(self, *args, **kwargs):
        functions = {
            'mocktest.ping': lambda: True,
            'mockgrains.get': lambda x: 'jerry',
        }
        minion_opts = salt.config.minion_config(os.path.join(TMP_CONF_DIR, 'minion'))
        render = salt.loader.render(minion_opts, functions)
        self.jinja = render.get('jinja')

    def tearDown(self):
        del self.jinja

    def render(self, tmpl_str, context=None):
        return self.jinja(tmpl_str, context=context or {}, from_str=True).read()

    def test_normlookup(self):
        '''
        Sanity-check the normal dictionary-lookup syntax for our stub function
        '''
        tmpl_str = '''Hello, {{ salt['mocktest.ping']() }}.'''

        with patch.object(SaltCacheLoader, 'file_client', Mock()):
            ret = self.render(tmpl_str)
        self.assertEqual(ret, 'Hello, True.')

    def test_dotlookup(self):
        '''
        Check calling a stub function using awesome dot-notation
        '''
        tmpl_str = '''Hello, {{ salt.mocktest.ping() }}.'''

        with patch.object(SaltCacheLoader, 'file_client', Mock()):
            ret = self.render(tmpl_str)
        self.assertEqual(ret, 'Hello, True.')

    def test_shadowed_dict_method(self):
        '''
        Check calling a stub function with a name that shadows a ``dict``
        method name
        '''
        tmpl_str = '''Hello, {{ salt.mockgrains.get('id') }}.'''

        with patch.object(SaltCacheLoader, 'file_client', Mock()):
            ret = self.render(tmpl_str)
        self.assertEqual(ret, 'Hello, jerry.')



# -*- coding: utf-8 -*-
from __future__ import absolute_import
import tarfile
import tempfile
import subprocess
import sys
import os

from tests.support.unit import TestCase

import salt.utils.files
import salt.utils.thin
try:
    import virtualenv
    HAS_VENV = True
except ImportError:
    HAS_VENV = False


class TestThinDir(TestCase):

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()

    def tearDown(self):
        salt.utils.files.rm_rf(self.tmpdir)

    def test_thin_dir(self):
        '''
        Test the thin dir to make sure salt-call can run

        Run salt call via a python in a new virtual environment to ensure
        salt-call has all dependencies needed.
        '''
        venv_dir = os.path.join(self.tmpdir, 'venv')
        virtualenv.create_environment(venv_dir)
        salt.utils.thin.gen_thin(self.tmpdir)
        thin_dir = os.path.join(self.tmpdir, 'thin')
        thin_archive = os.path.join(thin_dir, 'thin.tgz')
        tar = tarfile.open(thin_archive)
        tar.extractall(thin_dir)
        tar.close()
        bins = 'bin'
        if sys.platform == 'win32':
            bins = 'Scripts'
        cmd = [
            os.path.join(venv_dir, bins, 'python'),
            os.path.join(thin_dir, 'salt-call'),
            '--version',
        ]
        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        stdout, stderr = proc.communicate()
        proc.wait()
        assert proc.returncode == 0, (stdout, stderr, proc.returncode)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Anthony Shaw <anthonyshaw@apache.org>`
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    patch,
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON
)
import salt.modules.libcloud_loadbalancer as libcloud_loadbalancer

from libcloud.loadbalancer.base import BaseDriver, LoadBalancer, Algorithm, Member


class MockLBDriver(BaseDriver):
    def __init__(self):
        self._TEST_BALANCER = LoadBalancer(
            id='test_id', name='test_balancer',
            state=0,  # RUNNING
            ip='1.2.3.4',
            port=80, driver=self,
            extra={})
        self._TEST_MEMBER = Member(
            id='member_id', ip='12.3.4.5',
            port=443, balancer=self._TEST_BALANCER,
            extra=None)

    def get_balancer(self, balancer_id):
        assert balancer_id == 'test_id'
        return self._TEST_BALANCER

    def list_balancers(self):
        return [self._TEST_BALANCER]

    def list_protocols(self):
        return ['http', 'https']

    def create_balancer(self, name, port, protocol, algorithm, members):
        assert name == 'new_test_balancer'
        assert port == 80
        assert protocol == 'http'
        assert isinstance(algorithm, (Algorithm, int))
        assert isinstance(members, list)
        return self._TEST_BALANCER

    def destroy_balancer(self, balancer):
        assert balancer == self._TEST_BALANCER
        return True

    def balancer_attach_member(self, balancer, member):
        assert isinstance(balancer, LoadBalancer)
        assert isinstance(member, Member)
        assert member.id is None
        assert balancer.id == 'test_id'
        return self._TEST_MEMBER

    def balancer_detach_member(self, balancer, member):
        assert isinstance(balancer, LoadBalancer)
        assert isinstance(member, Member)
        assert member.id == 'member_id'
        assert balancer.id == 'test_id'
        return True

    def balancer_list_members(self, balancer):
        assert isinstance(balancer, LoadBalancer)
        assert balancer.id == 'test_id'
        return [self._TEST_MEMBER]


def get_mock_driver():
    return MockLBDriver()


@skipIf(NO_MOCK, NO_MOCK_REASON)
@patch('salt.modules.libcloud_loadbalancer._get_driver',
       MagicMock(return_value=MockLBDriver()))
class LibcloudLoadBalancerModuleTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        module_globals = {
            '__salt__': {
                'config.option': MagicMock(return_value={
                    'test': {
                        'driver': 'test',
                        'key': '2orgk34kgk34g'
                    }
                })
            }
        }
        if libcloud_loadbalancer.HAS_LIBCLOUD is False:
            module_globals['sys.modules'] = {'libcloud': MagicMock()}

        return {libcloud_loadbalancer: module_globals}

    def test_module_creation(self):
        client = libcloud_loadbalancer._get_driver('test')
        self.assertFalse(client is None)

    def test_init(self):
        with patch('salt.utils.compat.pack_dunder', return_value=False) as dunder:
            libcloud_loadbalancer.__init__(None)
            dunder.assert_called_with('salt.modules.libcloud_loadbalancer')

    def _validate_balancer(self, balancer):
        self.assertEqual(balancer['name'], 'test_balancer')

    def _validate_member(self, member):
        self.assertEqual(member['id'], 'member_id')
        self.assertEqual(member['ip'], '12.3.4.5')

    def test_list_balancers(self):
        balancers = libcloud_loadbalancer.list_balancers('test')
        self.assertEqual(len(balancers), 1)
        self._validate_balancer(balancers[0])

    def test_list_protocols(self):
        protocols = libcloud_loadbalancer.list_protocols('test')
        self.assertEqual(len(protocols), 2)
        self.assertTrue('http' in protocols)

    def test_create_balancer(self):
        balancer = libcloud_loadbalancer.create_balancer('new_test_balancer', 80, 'http', 'test')
        self._validate_balancer(balancer)

    def test_create_balancer_custom_algorithm(self):
        balancer = libcloud_loadbalancer.create_balancer('new_test_balancer', 80, 'http', 'test', algorithm='LEAST_CONNECTIONS')
        self._validate_balancer(balancer)

    def test_destroy_balancer(self):
        result = libcloud_loadbalancer.destroy_balancer('test_id', 'test')
        self.assertTrue(result)

    def test_get_balancer_by_name(self):
        balancer = libcloud_loadbalancer.get_balancer_by_name('test_balancer', 'test')
        self._validate_balancer(balancer)

    def test_get_balancer(self):
        balancer = libcloud_loadbalancer.get_balancer('test_id', 'test')
        self._validate_balancer(balancer)

    def test_balancer_attach_member(self):
        member = libcloud_loadbalancer.balancer_attach_member('test_id', '12.3.4.5', 443, 'test')
        self._validate_member(member)

    def test_balancer_detach_member(self):
        result = libcloud_loadbalancer.balancer_detach_member('test_id', 'member_id', 'test')
        self.assertTrue(result)

    def test_list_balancer_members(self):
        members = libcloud_loadbalancer.list_balancer_members('test_id', 'test')
        self._validate_member(members[0])



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.unit import skipIf
from tests.support.helpers import destructiveTest

# Import Salt Libs
import salt.utils.platform


@skipIf(not salt.utils.platform.is_windows(), 'Tests for only Windows')
class FirewallTest(ModuleCase):
    '''
    Validate windows firewall module
    '''
    def _pre_firewall_status(self, pre_run):
        post_run = self.run_function('firewall.get_config')
        network = ['Domain', 'Public', 'Private']
        # compare the status of the firewall before and after test
        # and re-enable or disable depending on status before test run
        for net in network:
            if post_run[net] != pre_run[net]:
                if pre_run[net]:
                    self.assertTrue(self.run_function('firewall.enable', profile=net))
                else:
                    self.assertTrue(self.run_function('firewall.disable', profile=net))

    @destructiveTest
    def test_firewall_get_config(self):
        '''
        test firewall.get_config
        '''
        pre_run = self.run_function('firewall.get_config')
        # ensure all networks are enabled then test status
        self.assertTrue(self.run_function('firewall.enable', profile='allprofiles'))
        ret = self.run_function('firewall.get_config')
        network = ['Domain', 'Public', 'Private']
        for net in network:
            self.assertTrue(ret[net])
        self._pre_firewall_status(pre_run)

    @destructiveTest
    def test_firewall_disable(self):
        '''
        test firewall.disable
        '''
        pre_run = self.run_function('firewall.get_config')
        network = 'Private'

        ret = self.run_function('firewall.get_config')[network]
        if not ret:
            self.assertTrue(self.run_function('firewall.enable', profile=network))

        self.assertTrue(self.run_function('firewall.disable', profile=network))
        ret = self.run_function('firewall.get_config')[network]
        self.assertFalse(ret)
        self._pre_firewall_status(pre_run)

    @destructiveTest
    def test_firewall_enable(self):
        '''
        test firewall.enable
        '''
        pre_run = self.run_function('firewall.get_config')
        network = 'Private'

        ret = self.run_function('firewall.get_config')[network]
        if ret:
            self.assertTrue(self.run_function('firewall.disable', profile=network))

        self.assertTrue(self.run_function('firewall.enable', profile=network))
        ret = self.run_function('firewall.get_config')[network]
        self.assertTrue(ret)
        self._pre_firewall_status(pre_run)

    def test_firewall_get_rule(self):
        '''
        test firewall.get_rule
        '''
        rule = 'Remote Event Log Management (NP-In)'

        ret = self.run_function('firewall.get_rule', [rule])
        checks = ['Private', 'LocalPort', 'RemotePort']
        for check in checks:
            self.assertIn(check, ret[rule])

    @destructiveTest
    def test_firewall_add_delete_rule(self):
        '''
        test firewall.add_rule and delete_rule
        '''
        rule = 'test rule'
        port = '8080'

        # test adding firewall rule
        add_rule = self.run_function('firewall.add_rule', [rule, port])
        ret = self.run_function('firewall.get_rule', [rule])
        self.assertIn(rule, ret[rule])
        self.assertIn(port, ret[rule])

        # test deleting firewall rule
        self.assertTrue(self.run_function('firewall.delete_rule', [rule, port]))
        ret = self.run_function('firewall.get_rule', [rule])
        self.assertNotIn(rule, ret)
        self.assertNotIn(port, ret)
        self.assertIn('No rules match the specified criteria.', ret)



# -*- coding: utf-8 -*-
'''
tests for pkgrepo states
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.mixins import SaltReturnAssertsMixin
from tests.support.unit import skipIf
from tests.support.helpers import (
    destructiveTest,
    requires_system_grains
)

# Import Salt libs
import salt.utils.platform

# Import 3rd-party libs
from salt.ext import six


@destructiveTest
@skipIf(salt.utils.platform.is_windows(), 'minion is windows')
class PkgrepoTest(ModuleCase, SaltReturnAssertsMixin):
    '''
    pkgrepo state tests
    '''
    @requires_system_grains
    def test_pkgrepo_01_managed(self, grains):
        '''
        Test adding a repo
        '''
        os_grain = self.run_function('grains.item', ['os'])['os']
        os_release_info = tuple(self.run_function('grains.item', ['osrelease_info'])['osrelease_info'])
        if os_grain == 'Ubuntu' and os_release_info >= (15, 10):
            self.skipTest(
                'The PPA used for this test does not exist for Ubuntu Wily'
                ' (15.10) and later.'
            )

        if grains['os_family'] == 'Debian':
            try:
                from aptsources import sourceslist
            except ImportError:
                self.skipTest(
                    'aptsources.sourceslist python module not found'
                )
        ret = self.run_function('state.sls', mods='pkgrepo.managed', timeout=120)
        # If the below assert fails then no states were run, and the SLS in
        # tests/integration/files/file/base/pkgrepo/managed.sls needs to be
        # corrected.
        self.assertReturnNonEmptySaltType(ret)
        for state_id, state_result in six.iteritems(ret):
            self.assertSaltTrueReturn(dict([(state_id, state_result)]))

    def test_pkgrepo_02_absent(self):
        '''
        Test removing the repo from the above test
        '''
        os_grain = self.run_function('grains.item', ['os'])['os']
        os_release_info = tuple(self.run_function('grains.item', ['osrelease_info'])['osrelease_info'])
        if os_grain == 'Ubuntu' and os_release_info >= (15, 10):
            self.skipTest(
                'The PPA used for this test does not exist for Ubuntu Wily'
                ' (15.10) and later.'
            )

        ret = self.run_function('state.sls', mods='pkgrepo.absent', timeout=120)
        # If the below assert fails then no states were run, and the SLS in
        # tests/integration/files/file/base/pkgrepo/absent.sls needs to be
        # corrected.
        self.assertReturnNonEmptySaltType(ret)
        for state_id, state_result in six.iteritems(ret):
            self.assertSaltTrueReturn(dict([(state_id, state_result)]))

    @requires_system_grains
    def test_pkgrepo_03_with_comments(self, grains):
        '''
        Test adding a repo with comments
        '''
        os_family = grains['os_family'].lower()

        if os_family in ('redhat',):
            kwargs = {
                'name': 'examplerepo',
                'baseurl': 'http://example.com/repo',
                'enabled': False,
                'comments': ['This is a comment']
            }
        elif os_family in ('debian',):
            self.skipTest('Debian/Ubuntu test case needed')
        else:
            self.skipTest("No test case for os_family '{0}'".format(os_family))

        try:
            # Run the state to add the repo
            ret = self.run_state('pkgrepo.managed', **kwargs)
            self.assertSaltTrueReturn(ret)

            # Run again with modified comments
            kwargs['comments'].append('This is another comment')
            ret = self.run_state('pkgrepo.managed', **kwargs)
            self.assertSaltTrueReturn(ret)
            ret = ret[next(iter(ret))]
            self.assertEqual(
                ret['changes'],
                {
                    'comments': {
                        'old': ['This is a comment'],
                        'new': ['This is a comment',
                                'This is another comment']
                    }
                }
            )

            # Run a third time, no changes should be made
            ret = self.run_state('pkgrepo.managed', **kwargs)
            self.assertSaltTrueReturn(ret)
            ret = ret[next(iter(ret))]
            self.assertFalse(ret['changes'])
            self.assertEqual(
                ret['comment'],
                "Package repo '{0}' already configured".format(kwargs['name'])
            )
        finally:
            # Clean up
            self.run_state('pkgrepo.absent', name=kwargs['name'])



# -*- coding: utf-8 -*-
'''
    :synopsis: Unit Tests for Advanced Packaging Tool module 'module.aptpkg'
    :platform: Linux
    :maturity: develop
    versionadded:: 2017.7.0
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals
import copy

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import MagicMock, patch, NO_MOCK, NO_MOCK_REASON

# Import Salt Libs
from salt.ext import six
from salt.exceptions import CommandExecutionError, SaltInvocationError
import salt.modules.aptpkg as aptpkg


APT_KEY_LIST = r'''
pub:-:1024:17:46181433FBB75451:1104433784:::-:::scSC:
fpr:::::::::C5986B4F1257FFA86632CBA746181433FBB75451:
uid:-::::1104433784::B4D41942D4B35FF44182C7F9D00C99AF27B93AD0::Ubuntu CD Image Automatic Signing Key <cdimage@ubuntu.com>:
'''

REPO_KEYS = {
    '46181433FBB75451': {
        'algorithm': 17,
        'bits': 1024,
        'capability': 'scSC',
        'date_creation': 1104433784,
        'date_expiration': None,
        'fingerprint': 'C5986B4F1257FFA86632CBA746181433FBB75451',
        'keyid': '46181433FBB75451',
        'uid': 'Ubuntu CD Image Automatic Signing Key <cdimage@ubuntu.com>',
        'uid_hash': 'B4D41942D4B35FF44182C7F9D00C99AF27B93AD0',
        'validity': '-'
    }
}

PACKAGES = {
    'wget': '1.15-1ubuntu1.14.04.2'
}

LOWPKG_FILES = {
    'errors': {},
    'packages': {
        'wget': [
            '/.',
            '/etc',
            '/etc/wgetrc',
            '/usr',
            '/usr/bin',
            '/usr/bin/wget',
            '/usr/share',
            '/usr/share/info',
            '/usr/share/info/wget.info.gz',
            '/usr/share/doc',
            '/usr/share/doc/wget',
            '/usr/share/doc/wget/MAILING-LIST',
            '/usr/share/doc/wget/NEWS.gz',
            '/usr/share/doc/wget/AUTHORS',
            '/usr/share/doc/wget/copyright',
            '/usr/share/doc/wget/changelog.Debian.gz',
            '/usr/share/doc/wget/README',
            '/usr/share/man',
            '/usr/share/man/man1',
            '/usr/share/man/man1/wget.1.gz',
        ]
    }
}

LOWPKG_INFO = {
    'wget': {
        'architecture': 'amd64',
        'description': 'retrieves files from the web',
        'homepage': 'http://www.gnu.org/software/wget/',
        'install_date': '2016-08-30T22:20:15Z',
        'maintainer': 'Ubuntu Developers <ubuntu-devel-discuss@lists.ubuntu.com>',
        'name': 'wget',
        'section': 'web',
        'source': 'wget',
        'version': '1.15-1ubuntu1.14.04.2'
    }
}

APT_Q_UPDATE = '''
Get:1 http://security.ubuntu.com trusty-security InRelease [65 kB]
Get:2 http://security.ubuntu.com trusty-security/main Sources [120 kB]
Get:3 http://security.ubuntu.com trusty-security/main amd64 Packages [548 kB]
Get:4 http://security.ubuntu.com trusty-security/main i386 Packages [507 kB]
Hit http://security.ubuntu.com trusty-security/main Translation-en
Fetched 1240 kB in 10s (124 kB/s)
Reading package lists...
'''

APT_Q_UPDATE_ERROR = '''
Err http://security.ubuntu.com trusty InRelease

Err http://security.ubuntu.com trusty Release.gpg
Unable to connect to security.ubuntu.com:http:
Reading package lists...
W: Failed to fetch http://security.ubuntu.com/ubuntu/dists/trusty/InRelease

W: Failed to fetch http://security.ubuntu.com/ubuntu/dists/trusty/Release.gpg  Unable to connect to security.ubuntu.com:http:

W: Some index files failed to download. They have been ignored, or old ones used instead.
'''

AUTOREMOVE = '''
Reading package lists... Done
Building dependency tree
Reading state information... Done
0 upgraded, 0 newly installed, 0 to remove and 0 not upgraded.
'''

UPGRADE = '''
Reading package lists...
Building dependency tree...
Reading state information...
0 upgraded, 0 newly installed, 0 to remove and 0 not upgraded.
'''

UNINSTALL = {
    'tmux': {
        'new': six.text_type(),
        'old': '1.8-5'
    }
}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class AptPkgTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.aptpkg
    '''

    def setup_loader_modules(self):
        return {aptpkg: {}}

    def test_version(self):
        '''
        Test - Returns a string representing the package version or an empty string if
        not installed.
        '''
        version = LOWPKG_INFO['wget']['version']
        mock = MagicMock(return_value=version)
        with patch.dict(aptpkg.__salt__, {'pkg_resource.version': mock}):
            self.assertEqual(aptpkg.version(*['wget']), version)

    def test_upgrade_available(self):
        '''
        Test - Check whether or not an upgrade is available for a given package.
        '''
        with patch('salt.modules.aptpkg.latest_version',
                   MagicMock(return_value='')):
            self.assertFalse(aptpkg.upgrade_available('wget'))

    def test_add_repo_key(self):
        '''
        Test - Add a repo key.
        '''
        with patch('salt.modules.aptpkg.get_repo_keys',
                   MagicMock(return_value=REPO_KEYS)):
            mock = MagicMock(return_value={
                'retcode': 0,
                'stdout': 'OK'
            })
            with patch.dict(aptpkg.__salt__, {'cmd.run_all': mock}):
                self.assertTrue(aptpkg.add_repo_key(keyserver='keyserver.ubuntu.com',
                                                    keyid='FBB75451'))

    def test_add_repo_key_failed(self):
        '''
        Test - Add a repo key using incomplete input data.
        '''
        with patch('salt.modules.aptpkg.get_repo_keys',
                   MagicMock(return_value=REPO_KEYS)):
            kwargs = {'keyserver': 'keyserver.ubuntu.com'}
            mock = MagicMock(return_value={
                'retcode': 0,
                'stdout': 'OK'
            })
            with patch.dict(aptpkg.__salt__, {'cmd.run_all': mock}):
                self.assertRaises(SaltInvocationError, aptpkg.add_repo_key, **kwargs)

    def test_get_repo_keys(self):
        '''
        Test - List known repo key details.
        '''
        mock = MagicMock(return_value={
            'retcode': 0,
            'stdout': APT_KEY_LIST
        })
        with patch.dict(aptpkg.__salt__, {'cmd.run_all': mock}):
            self.assertEqual(aptpkg.get_repo_keys(), REPO_KEYS)

    def test_file_dict(self):
        '''
        Test - List the files that belong to a package, grouped by package.
        '''
        mock = MagicMock(return_value=LOWPKG_FILES)
        with patch.dict(aptpkg.__salt__, {'lowpkg.file_dict': mock}):
            self.assertEqual(aptpkg.file_dict('wget'), LOWPKG_FILES)

    def test_file_list(self):
        '''
        Test - List the files that belong to a package.
        '''
        files = {
            'errors': LOWPKG_FILES['errors'],
            'files': LOWPKG_FILES['packages']['wget'],
        }
        mock = MagicMock(return_value=files)
        with patch.dict(aptpkg.__salt__, {'lowpkg.file_list': mock}):
            self.assertEqual(aptpkg.file_list('wget'), files)

    def test_get_selections(self):
        '''
        Test - View package state from the dpkg database.
        '''
        selections = {'install': ['wget']}
        mock = MagicMock(return_value='wget\t\t\t\t\t\tinstall')
        with patch.dict(aptpkg.__salt__, {'cmd.run_stdout': mock}):
            self.assertEqual(aptpkg.get_selections('wget'), selections)

    def test_info_installed(self):
        '''
        Test - Return the information of the named package(s) installed on the system.
        '''
        names = {
            'group': 'section',
            'packager': 'maintainer',
            'url': 'homepage'
        }

        installed = copy.deepcopy(LOWPKG_INFO)
        for name in names:
            if installed['wget'].get(names[name], False):
                installed['wget'][name] = installed['wget'].pop(names[name])

        mock = MagicMock(return_value=LOWPKG_INFO)
        with patch.dict(aptpkg.__salt__, {'lowpkg.info': mock}):
            self.assertEqual(aptpkg.info_installed('wget'), installed)

    def test_owner(self):
        '''
        Test - Return the name of the package that owns the file.
        '''
        paths = ['/usr/bin/wget']
        mock = MagicMock(return_value='wget: /usr/bin/wget')
        with patch.dict(aptpkg.__salt__, {'cmd.run_stdout': mock}):
            self.assertEqual(aptpkg.owner(*paths), 'wget')

    def test_refresh_db(self):
        '''
        Test - Updates the APT database to latest packages based upon repositories.
        '''
        refresh_db = {
            'http://security.ubuntu.com trusty-security InRelease': True,
            'http://security.ubuntu.com trusty-security/main Sources': True,
            'http://security.ubuntu.com trusty-security/main Translation-en': None,
            'http://security.ubuntu.com trusty-security/main amd64 Packages': True,
            'http://security.ubuntu.com trusty-security/main i386 Packages': True
        }
        mock = MagicMock(return_value={
            'retcode': 0,
            'stdout': APT_Q_UPDATE
        })
        with patch('salt.utils.pkg.clear_rtag', MagicMock()):
            with patch.dict(aptpkg.__salt__, {'cmd.run_all': mock}):
                self.assertEqual(aptpkg.refresh_db(), refresh_db)

    def test_refresh_db_failed(self):
        '''
        Test - Update the APT database using unreachable repositories.
        '''
        kwargs = {'failhard': True}
        mock = MagicMock(return_value={
            'retcode': 0,
            'stdout': APT_Q_UPDATE_ERROR
        })
        with patch('salt.utils.pkg.clear_rtag', MagicMock()):
            with patch.dict(aptpkg.__salt__, {'cmd.run_all': mock}):
                self.assertRaises(CommandExecutionError, aptpkg.refresh_db, **kwargs)

    def test_autoremove(self):
        '''
        Test - Remove packages not required by another package.
        '''
        with patch('salt.modules.aptpkg.list_pkgs',
                   MagicMock(return_value=PACKAGES)):
            patch_kwargs = {
                '__salt__': {
                    'config.get': MagicMock(return_value=True),
                    'cmd.run': MagicMock(return_value=AUTOREMOVE)
                }
            }
            with patch.multiple(aptpkg, **patch_kwargs):
                self.assertEqual(aptpkg.autoremove(), dict())
                self.assertEqual(aptpkg.autoremove(purge=True), dict())
                self.assertEqual(aptpkg.autoremove(list_only=True), list())
                self.assertEqual(aptpkg.autoremove(list_only=True, purge=True), list())

    def test_remove(self):
        '''
        Test - Remove packages.
        '''
        with patch('salt.modules.aptpkg._uninstall',
                   MagicMock(return_value=UNINSTALL)):
            self.assertEqual(aptpkg.remove(name='tmux'), UNINSTALL)

    def test_purge(self):
        '''
        Test - Remove packages along with all configuration files.
        '''
        with patch('salt.modules.aptpkg._uninstall',
                   MagicMock(return_value=UNINSTALL)):
            self.assertEqual(aptpkg.purge(name='tmux'), UNINSTALL)

    def test_upgrade(self):
        '''
        Test - Upgrades all packages.
        '''
        with patch('salt.utils.pkg.clear_rtag', MagicMock()):
            with patch('salt.modules.aptpkg.list_pkgs',
                       MagicMock(return_value=UNINSTALL)):
                mock_cmd = MagicMock(return_value={
                    'retcode': 0,
                    'stdout': UPGRADE
                })
                patch_kwargs = {
                    '__salt__': {
                        'config.get': MagicMock(return_value=True),
                        'cmd.run_all': mock_cmd
                    }
                }
                with patch.multiple(aptpkg, **patch_kwargs):
                    self.assertEqual(aptpkg.upgrade(), dict())



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Anthony Shaw <anthonyshaw@apache.org>`
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON,
    patch
)

import tests.support.napalm as napalm_test_support
import salt.proxy.napalm as napalm_proxy  # NOQA


TEST_OPTS = {
    'proxytype': 'napalm',
    'driver': 'junos',
    'host': 'core05.nrt02'
}


def mock_get_device(opts, *args, **kwargs):
    assert opts == TEST_OPTS
    return {
        'DRIVER': napalm_test_support.MockNapalmDevice(),
        'UP': True
    }


@skipIf(NO_MOCK, NO_MOCK_REASON)
@patch('salt.utils.napalm.get_device', mock_get_device)
class NapalmProxyTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        module_globals = {
            '__salt__': {
                'config.option': MagicMock(return_value={
                    'test': {
                        'driver': 'test',
                        'key': '2orgk34kgk34g'
                    }
                })
            }
        }
        module_globals['napalm_base'] = MagicMock()
        return {napalm_proxy: module_globals}

    def test_init(self):
        ret = napalm_proxy.init(TEST_OPTS)
        assert ret is True

    def test_alive(self):
        ret = napalm_proxy.alive(TEST_OPTS)
        assert ret is True

    def test_ping(self):
        napalm_proxy.init(TEST_OPTS)
        ret = napalm_proxy.ping()
        assert ret is True

    def test_initialized(self):
        napalm_proxy.init(TEST_OPTS)
        ret = napalm_proxy.initialized()
        assert ret is True

    def test_get_device(self):
        napalm_proxy.init(TEST_OPTS)
        ret = napalm_proxy.get_device()
        assert ret['UP'] is True

    def test_get_grains(self):
        napalm_proxy.init(TEST_OPTS)
        ret = napalm_proxy.get_grains()
        assert ret['out'] == napalm_test_support.TEST_FACTS

    def test_grains_refresh(self):
        napalm_proxy.init(TEST_OPTS)
        ret = napalm_proxy.grains_refresh()
        assert ret['out'] == napalm_test_support.TEST_FACTS

    def test_fns(self):
        ret = napalm_proxy.fns()
        assert 'details' in ret.keys()

    def test_shutdown(self):
        ret = napalm_proxy.shutdown(TEST_OPTS)
        assert ret is True

    def test_call(self):
        napalm_proxy.init(TEST_OPTS)
        ret = napalm_proxy.call('get_arp_table')
        assert ret['out'] == napalm_test_support.TEST_ARP_TABLE
